@@ -20,8 +20,8 @@ public:
   {
     registerOptions();
   }
-    
-  char *name()
+
+  const char *name()
   {
     return "_facets";
   }
@@ -36,11 +36,11 @@ public:
     for(IntegerVectorList::const_iterator i=normals.begin();i!=normals.end();i++)
       if(isFacet(normals,i))
         {
-	  facets.push_back(*i);
+    	  facets.push_back(*i);
         }
-    
+
     AsciiPrinter(Stdout).printVectorList(facets);
- 
+
     log1 fprintf(Stderr,"Rank:%i\n",rankOfMatrix(facets));
 
     return 0;
