@@ -10,17 +10,19 @@
 class LatticeIdealApplication : public GFanApplication
 {
   SimpleOption toricOption;
+  SimpleOption justConvertOption;
 public:
   bool includeInDefaultInstallation()
   {
-    return false;
+    return true;
   }
   LatticeIdealApplication():
-    toricOption("-t","Compute the toric ideal of the matrix whose rows are given on the input instead.")
+    toricOption("-t","Compute the toric ideal of the matrix whose rows are given on the input instead."),
+	justConvertOption("--convert","Does not do any computation, but just converts the vectors to binomials.")
   {
     registerOptions();
   }
-  char *name()
+  const char *name()
   {
     return "_latticeideal";
   }
@@ -34,7 +36,11 @@ public:
     if(toricOption.getValue())
       A=latticeKernelOfTransposed(A).transposed();
 
-    IntegerVectorList b=latticeIdealRevLex(A);
+    IntegerVectorList b;
+    if(justConvertOption.getValue())
+    	b=A.transposed().getRows();
+    else
+    	b=latticeIdealRevLex(A);
 
     PolynomialRing theRing(Q,A.getHeight());
     PolynomialSet g(theRing);
@@ -47,7 +53,7 @@ public:
   }
   const char *helpText()
   {
-    return "This program computes the lattice ideal of the lattice. The input is a list of generators for the lattice.\n";
+    return "This program computes the lattice ideal of a lattice. The input is a list of generators for the lattice.\n";
   }
 };
 
