@@ -14,7 +14,7 @@ public:
   {
     registerOptions();
   }
-  char *name()
+  const char *name()
   {
     return "_multiplymatrix";
   }
