@@ -24,14 +24,14 @@ public:
   }
   const char *helpText()
   {
-    return "This program takes a matrix as input and checks if it satisfies Scarf's generality conditions.\n";
+    return "This program takes a matrix as input and checks if it satisfies Scarf's generality conditions. The rows of the matrix are listed on the input. The A1 condition is that there exists a strictly poistive vector in the co-kernel of the matrix. The A2 condition is that te maximal minors of the matrix are non-zero. A3\n";
   }
   ScarfIsGenericApplication()
   {
     registerOptions();
   }
-    
-  char *name()
+
+  const char *name()
   {
     return "_scarf_isgeneric";
   }
@@ -39,7 +39,7 @@ public:
   {
     LpSolver::printList(Stdout);
     lpSetSolver("cddgmp");
-    
+
     FileParser P(Stdin);
 
     IntegerVectorList ivl=P.parseIntegerVectorList();
@@ -54,6 +54,8 @@ public:
 	N=orientedNeighbours(N,-A[0]);
 	Q.printVectorList(N);
 	fprintf(Stdout,"A2 satisfied:%i\n",satisfiesA2(A));
+	for(int i=0;i<A.getHeight();i++)
+	  fprintf(Stdout,"A3(%i) satisfied:%i\n",i,satisfiesA3i(A,i,&N));
 	fprintf(Stdout,"A3 satisfied:%i\n",satisfiesA3(A,&N));
       }
     else
