@@ -15,7 +15,7 @@ public:
   {
     registerOptions();
   }
-  char *name()
+  const char *name()
   {
     return "_transposematrix";
   }
