@@ -12,8 +12,13 @@
 #include "tropical2.h"
 #include "tropicalbasis.h"
 #include "polyhedralcone.h"
+#include "polyhedralfan.h"
 #include "multiplicity.h"
 #include "log.h"
+#include "restrictedautoreduction.h"
+#include "groebnerengine.h"
+
+#include <iostream>
 
 static Polynomial wallPolynomial(Polynomial const &p, Subspace const &subspace)
   // This routine should be deleted since it is inefficient
@@ -58,6 +63,9 @@ BergmanFan bergmanRayIntersection(PolynomialSet const &idealGroebnerBasis)
 
   int n=idealGroebnerBasis.numberOfVariablesInRing();
 
+  //  log0 AsciiPrinter(Stderr).printPolynomialSet(idealGroebnerBasis);
+
+
   int d=krullDimension(idealGroebnerBasis); //There should be a better way of doing this
   PolyhedralFan bergmanFan(n);
   PolynomialSet tropBasis=tropicalBasisOfCurve(n,idealGroebnerBasis,&bergmanFan,d-1);
@@ -70,11 +78,16 @@ BergmanFan bergmanRayIntersection(PolynomialSet const &idealGroebnerBasis)
   //  AsciiPrinter(Stderr).printVectorList(rays);
   //  fprintf(Stderr,"---------------------------------------------------------\n");
 
+
+  log2  cerr<<"BBLABL";
   for(IntegerVectorList::const_iterator i=rays.begin();i!=rays.end();i++)
     {
+      log2 cerr<<"1";
       PolynomialSet g=idealGroebnerBasis;
-      buchberger(&g,WeightReverseLexicographicTermOrder(*i));
+      g=GE_groebnerBasis(g,WeightReverseLexicographicTermOrder(*i),true);
+      //      buchberger(&g,WeightReverseLexicographicTermOrder(*i));
       PolynomialSet cg=initialFormsAssumeMarked(g,*i);
+      log2 cerr<<"2";
  
       bool inList=false;
       for(BergmanFan::MaximalConeList::const_iterator j=bfan.cones.begin();j!=bfan.cones.end();j++)
@@ -85,17 +98,19 @@ BergmanFan bergmanRayIntersection(PolynomialSet const &idealGroebnerBasis)
 	      break;
 	    }
 	}
+      log2 cerr<<"3";
       if(!inList)
 	{
-	  bfan.cones.push_back(BergmanFan::MaximalCone(cg,g,maximalConeLabel++));
+	  bfan.cones.push_back(BergmanFan::MaximalCone(cg,g,true,maximalConeLabel++));
 	}
+      log2 cerr<<"4";
     }
+  log2 cerr<<"BBLAB  END";
   //  AsciiPrinter temp(Stderr);
   //  bfan.print(temp);
   return bfan;
 }
 
-
 BergmanFan bergmanRay(PolynomialSet const &idealGroebnerBasis)
   // Input ideal is assumed to be homogeneous with respect to a positive vector
   // Input ideal is assumed not to contain a monomial
@@ -196,7 +211,7 @@ BergmanFan bergmanRay(PolynomialSet const &idealGroebnerBasis)
 		}
 	      if(!inList)
 		{
-		  bfan.cones.push_back(BergmanFan::MaximalCone(cg,*g,maximalConeLabel++));
+		  bfan.cones.push_back(BergmanFan::MaximalCone(cg,*g,true,maximalConeLabel++));
 		}
 	    }
 	  //	  else
@@ -220,14 +235,14 @@ BergmanFan bergmanRay(PolynomialSet const &idealGroebnerBasis)
 }
 
 
-static bool staticInOrbit(PolynomialSet const &groebnerBasis1, PolynomialSet const &groebnerBasis2, SymmetryGroup const &s)
+/*static bool staticInOrbit(PolynomialSet const &groebnerBasis1, PolynomialSet const &groebnerBasis2, SymmetryGroup const &s)
 {
   for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
     if(areIdealsEqual(SymmetryGroup::permutePolynomialSet(groebnerBasis2,*j),groebnerBasis1))return true;
   return false;
-}
+  }*/
 
-static bool staticPermutationFixesCone(PolynomialSet const &groebnerBasis, IntegerVector const &v)
+/*static bool staticPermutationFixesCone(PolynomialSet const &groebnerBasis, IntegerVector const &v)
 { // Cone is fixed iff the cone ideal is fixed.
   PolynomialSet q(groebnerBasis.getRing());
   for(PolynomialSet::const_iterator i=groebnerBasis.begin();i!=groebnerBasis.end();i++)
@@ -235,10 +250,10 @@ static bool staticPermutationFixesCone(PolynomialSet const &groebnerBasis, Integ
       q.push_back(SymmetryGroup::permutePolynomial(*i,v));
     }
   return areIdealsEqual(q,groebnerBasis);
-}
+  }*/
 
 
-static int staticOrbitSize(PolynomialSet const &groebnerBasis, SymmetryGroup const &s)
+/*static int staticOrbitSize(PolynomialSet const &groebnerBasis, SymmetryGroup const &s)
 {
   int groupSize=s.elements.size();
   
@@ -249,6 +264,12 @@ static int staticOrbitSize(PolynomialSet const &groebnerBasis, SymmetryGroup con
   //  fprintf(Stderr,"groupSize = %i, numFixed = %i\n",groupSize,numFixed);
   return groupSize/numFixed;
 }
+*/
+
+
+
+/** Objects of this class represent ConeOrbits of facet of a tropical variety.
+ */
 
 class ConeOrbit
 {
@@ -257,21 +278,80 @@ public:
   PolynomialSet coneGroebnerBasis;
   PolynomialSet idealGroebnerBasis;
   int label;
-  PolynomialSetList markedFacets;
+  //  PolynomialSetList markedFacets;
+  PolyhedralCone theCone;
+  //  IntegerVectorList markedFacets; //marked facets of dim d cone. The facet of a facet is a ridge of the tropical variety.
+  IntegerVector stableRay; //stable relative interior point
+  IntegerVectorList unMarkedFacetNormals;
 
   ConeOrbit(const SymmetryGroup &s_, PolynomialSet const &coneGroebnerBasis_, PolynomialSet const &idealGroebnerBasis_, int label_):
     coneGroebnerBasis(coneGroebnerBasis_),
     idealGroebnerBasis(idealGroebnerBasis_),
     label(label_),
-    s(s_)
-  { 
-  }
-  void markFacet(PolynomialSet const &f)
+    s(s_),
+    theCone(fastNormals(wallInequalities(idealGroebnerBasis_)),//wallFlipableNormals(idealGroebnerBasis_,false),
+	    wallInequalities(coneGroebnerBasis_),
+	    idealGroebnerBasis_.getRing().getNumberOfVariables())
   {
-    markedFacets.push_back(f);
+    /*
+log0 AsciiPrinter(Stderr).printPolynomialSet(coneGroebnerBasis_);
+log0 AsciiPrinter(Stderr).printPolynomialSet(idealGroebnerBasis_);
+
+    log0 fprintf(Stderr,"inequalities:\n");
+    log0 AsciiPrinter(Stderr).printVectorList(wallInequalities(coneGroebnerBasis_));
+    log0 fprintf(Stderr,"equalities:\n");
+    log0 AsciiPrinter(Stderr).printVectorList(wallFlipableNormals(idealGroebnerBasis_,false));
+
+    AsciiPrinter P(Stderr);
+    log0 theCone.print(&P);
+    */
+    theCone.findFacets();
+    //    theCone.canonicalize();//REMOVE ME JAN 2009
+    stableRay=PolyhedralFan::stableRay(theCone,&s);
+    unMarkedFacetNormals=theCone.getHalfSpaces();
+  }
+  IntegerVector getStableRay()const
+  {
+    return stableRay;
+  }
+  bool doesPermutationFixCone(IntegerVector const &v)const
+  { // Cone is fixed iff stableRay is fixed.
+    return (SymmetryGroup::compose(v,stableRay)==stableRay);
+  }
+  int orbitSize()const
+  {
+    int groupSize=s.elements.size();
+    
+    int numFixed=0;
+    for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
+      if(doesPermutationFixCone(*j))numFixed++;
+    
+    //  fprintf(Stderr,"groupSize = %i, numFixed = %i\n",groupSize,numFixed);
+    return groupSize/numFixed;
+  }
+
+  void markFacet(IntegerVector const &relIntRidgeVector)
+  {
+    for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
+      if(doesPermutationFixCone(*j))
+	{
+	  IntegerVector relIntRidgeVector2=SymmetryGroup::compose(*j,relIntRidgeVector);
+	  {
+	    for(IntegerVectorList::iterator i=unMarkedFacetNormals.begin();i!=unMarkedFacetNormals.end();)
+	      if(dotLong(*i,relIntRidgeVector2)==0)
+		{
+		  IntegerVectorList::iterator temp=i;
+		  temp++;
+		  unMarkedFacetNormals.erase(i);
+		  i=temp;
+		}
+	      else
+		i++;
+	  }
+	}
   }
   
-  bool containsAndMark(PolynomialSet const &coneGroebnerBasis_, PolynomialSet const &facetIdeal, IntegerVector *labelPermutation)
+  /*  bool containsAndMark(PolynomialSet const &coneGroebnerBasis_, PolynomialSet const &facetIdeal, IntegerVector *labelPermutation)
   {
     for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
       if(areIdealsEqual(coneGroebnerBasis,SymmetryGroup::permutePolynomialSet(coneGroebnerBasis_,*j)))
@@ -290,31 +370,73 @@ public:
 	  return true;
 	}
     return false;
+    }*/
+  /* Alternative using only geometric information.
+   */
+  bool containsAndMark(IntegerVector const &relIntFacetVector, IntegerVector const &relIntRidgeVector, IntegerVector *labelPermutation)
+  {
+    for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
+      if(theCone.contains(SymmetryGroup::compose(*j,relIntFacetVector)))
+	{
+	  markFacet(SymmetryGroup::compose(*j,relIntRidgeVector));
+	  if(labelPermutation)*labelPermutation=*j;
+	  return true;
+	}
+    return false;
   }
-  bool isMarkedFacet(PolynomialSet const &f)
+  bool hasUnmarkedFacet()const
+  {
+    return(!unMarkedFacetNormals.empty());    
+  }
+  IntegerVector getUnmarkedFacet()const
+  {
+    assert(!unMarkedFacetNormals.empty());
+    return *unMarkedFacetNormals.begin();
+  }
+  /*  bool isMarkedFacet(PolynomialSet const &f)
   {
     for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
       if(staticPermutationFixesCone(coneGroebnerBasis,*j))
 	for(PolynomialSetList::const_iterator i=markedFacets.begin();i!=markedFacets.end();i++)
 	  if(areIdealsEqual(SymmetryGroup::permutePolynomialSet(f,*j),*i))return true;
     return false;
-  }
-  int orbitSize()
-  {
-    return staticOrbitSize(coneGroebnerBasis,s);
-  }
+    }*/
   void print(AsciiPrinter &p)const
   {
     p.printString("ConeOrbit{\n");
     p.printInteger(label);
+    p.printNewLine();
+    IntegerVector v=getStableRay();
+    /*    if(v.size()==25)
+      {
+	p.printVector(v.subvector(0,5));p.printNewLine();
+	p.printVector(v.subvector(5,10));p.printNewLine();
+	p.printVector(v.subvector(10,15));p.printNewLine();
+	p.printVector(v.subvector(15,20));p.printNewLine();
+	p.printVector(v.subvector(20,25));p.printNewLine();	
+	}*/
+    p.printVector(v);
+    p.printNewLine();
+    p.printString("Orbit size:");
+    p.printInteger(orbitSize());
+
     p.printString("\nConeIdeal:\n");
     p.printPolynomialSet(coneGroebnerBasis);
     p.printString("\nFullIdeal:\n");
     p.printPolynomialSet(idealGroebnerBasis);
-    p.printString("Marked facets:\n");
-    p.printPolynomialSetList(markedFacets);
+    //    p.printString("Unmarked facet normals:\n");
+    //    p.printVectorList(unMarkedFacetNormals);
+    theCone.print(&p);
     p.printString("}ConeOrbit\n");
   }
+  IntegerVectorList getEquations()const
+  {
+    return theCone.getEquations();
+  }
+  IntegerVectorList getInequalities()const
+  {
+    return theCone.getHalfSpaces();
+  }
 };
 
 class ConeOrbitContainer
@@ -353,7 +475,8 @@ public:
       }
     p.printString("}OrbitList\n");
   }
-  bool containsAndMark(PolynomialSet const &coneGroebnerBasis, PolynomialSet const &facetIdeal, int *label, IntegerVector *labelPermutation)
+  bool containsAndMark(IntegerVector const &relIntFacetVector, IntegerVector const &relIntRidgeVector, int *label, IntegerVector *labelPermutation)
+  //  bool containsAndMark(PolynomialSet const &coneGroebnerBasis, PolynomialSet const &facetIdeal, int *label, IntegerVector *labelPermutation)
   {
     //    fprintf(Stderr,"listlength:%i",l.size());
     //  int I=0;
@@ -367,7 +490,8 @@ public:
 	AsciiPrinter(Stderr).printPolynomialSet(coneGroebnerBasis);
 	AsciiPrinter(Stderr).printPolynomialSet(i->coneGroebnerBasis);
 	*/
-	if(i->containsAndMark(coneGroebnerBasis,facetIdeal,labelPermutation))
+	//	if(i->containsAndMark(coneGroebnerBasis,facetIdeal,labelPermutation))
+	if(i->containsAndMark(relIntFacetVector,relIntRidgeVector,labelPermutation))
 	   //	if(areIdealsEqual(coneGroebnerBasis,i->coneGroebnerBasis))// this could be slow!
 	  {
 	    *label=i->label;
@@ -382,6 +506,11 @@ public:
   }
 };
 
+static int staticOrbitSize(PolynomialSet const &coneGroebnerBasis, PolynomialSet const &idealGroebnerBasis, SymmetryGroup const &s)
+{
+  return ConeOrbit(s, coneGroebnerBasis, idealGroebnerBasis, -1).orbitSize();
+}
+
 BergmanFan bergman(PolynomialSet const &coneGroebnerBasis1, PolynomialSet const &idealGroebnerBasis1, SymmetryGroup const *symmetryGroup)
 {
   PolynomialRing theRing=coneGroebnerBasis1.getRing();
@@ -413,46 +542,29 @@ BergmanFan bergman(PolynomialSet const &coneGroebnerBasis1, PolynomialSet const
       log1 fprintf(Stderr,"Size of active set: %i, Completed: %i\n",active.size(),ret.cones.size());
       log1 fprintf(Stderr,"-------------------------------------\n");
       AsciiPrinter p(Stderr);
-      //      fprintf(Stderr,"----------------Active--------------------\n");
-      //      active.print(p);
-      //      ret.print(p);
+      
       
       ConeOrbit &current=active.front();
-
-      assert(current.idealGroebnerBasis.isMarked());
-      assert(current.coneGroebnerBasis.isMarked());
-
-      /*      fprintf(Stderr,"ConeGroebnerBasis:\n");
-      AsciiPrinter(Stderr).printPolynomialSet(current.coneGroebnerBasis);
-      fprintf(Stderr,"\n");
-      fprintf(Stderr,"IdealGroebnerBasis:\n");
-      AsciiPrinter(Stderr).printPolynomialSet(current.idealGroebnerBasis);
-      fprintf(Stderr,"\n");
-      */
-      IntegerVectorList equalities=wallInequalities(current.coneGroebnerBasis);
-
-      /*fprintf(Stderr,"Perp:\n");
-      AsciiPrinter(Stderr).printVectorList(facePerp);
-      AsciiPrinter(Stderr).printPolynomialSet(current.idealGroebnerBasis);
-      */
-      IntegerVectorList normals=wallFlipableNormals(current.idealGroebnerBasis,false);
-
-
-
-      /*fprintf(Stderr,"Normals:\n");
-      AsciiPrinter(Stderr).printVectorList(normals);
+      
+      /*      AsciiPrinter p2(stdout);//REMOVE THESE THREE LINES
+      current.print(p2);
+      fflush(stdout);
       */
 
+      //      IntegerVectorList equalities=wallInequalities(current.coneGroebnerBasis);
+
+      //      IntegerVectorList normals=wallFlipableNormals(current.idealGroebnerBasis,false);
+      /*
       {
 	PolyhedralCone p(normals,equalities);
 	p.findFacets();
 	isSimplicial&=p.isSimplicial();
-      }
-
-      removeRedundantRows(&normals,&equalities,true);//IS THIS RIGHT?
-
-      IntegerVectorList facePerp=equalities;
+	}HUSK AT INDSÆTTE DETTE IGEN!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+      */
+      //      removeRedundantRows(&normals,&equalities,true);//IS THIS RIGHT?
 
+      IntegerVectorList facePerp=current.getEquations();//equalities;
+	   /*
       int numberOfNormals=normals.size();
       int numberOfEqualities=equalities.size();
       IntegerVectorList inequalities=equalities;
@@ -462,98 +574,143 @@ BergmanFan bergman(PolynomialSet const &coneGroebnerBasis1, PolynomialSet const
       for(int i=0;i<numberOfEqualities;i++)
 	equalitySet[i+numberOfNormals]=1;
 
-      //  fprintf(Stderr,"Inequalities:\n");
-      //  AsciiPrinter(Stderr).printVectorList(inequalities);
-      //  fprintf(Stderr,"EqualitySet:\n");
-      //  AsciiPrinter(Stderr).printVector(equalitySet);
       IntegerVectorList::const_iterator i=inequalities.begin();
+      */
       int numberOfValidFacets=0;
       int numberOfAlreadyMarkedFacets=0;
-      for(int I=0;I<numberOfNormals;I++)
+      while(current.hasUnmarkedFacet())
+	//for(int I=0;I<numberOfNormals;I++)
 	{
-	  //AsciiPrinter(Stderr).printVector(*i);
-	  equalitySet[I]=1;
-	  //AsciiPrinter(Stderr).printVector(equalitySet);
-	  //  fprintf(Stderr,(hasInteriorPoint(inequalities,true,&equalitySet))?"TRUE\n":"FALSE\n");
-	  if(hasInteriorPoint(inequalities,true,&equalitySet))
+
+	  //      log1 active.print(p);
+	  IntegerVector theUnmarkedFacetNormal=current.getUnmarkedFacet();
+
+
+	  IntegerVector relIntRidgeVector;
+	  {
+	    IntegerVectorList equations=current.getEquations();
+	    IntegerVectorList inequalities=current.getInequalities();
+	    equations.push_back(theUnmarkedFacetNormal);
+	    PolyhedralCone C(inequalities,equations,n);
+	    relIntRidgeVector=C.getRelativeInteriorPoint();
+	  }
+	  //log0 fprintf(Stderr,"RIDGE:");
+	  //log0 p.printVector(relIntRidgeVector);
+
+	  //	  equalitySet[I]=1;
+	  //	  if(hasInteriorPoint(inequalities,true,&equalitySet))
 	    {
 	      // compute initial ideal
 	      PolynomialSet initialIdeal(theRing);
 	      int oldDim=rankOfMatrix(facePerp);
-	      facePerp.push_back(*i);
+
+	      //facePerp.push_back(*i);
+	      facePerp.push_back(theUnmarkedFacetNormal);
 	      int newDim=rankOfMatrix(facePerp);
-	      //	      for(PolynomialSet::const_iterator k=current.idealGroebnerBasis.begin();k!=current.idealGroebnerBasis.end();k++)
-	      //		initialIdeal.push_back(wallPolynomial(*k,facePerp));
 	      initialIdeal=wallIdeal(current.idealGroebnerBasis,Subspace(facePerp));
 	      facePerp.pop_back();
 
+
 	      if(oldDim!=newDim)
 		{
 		  numberOfValidFacets++;
-		  if(!current.isMarkedFacet(initialIdeal))
+		  //  if(!current.isMarkedFacet(initialIdeal))
 		    {
-		      //      fprintf(Stderr,"Computing Bergman fan of initial ideal:\n");
-		      //      AsciiPrinter(Stderr).printPolynomialSet(initialIdeal);
-
 		      BergmanFan b=useFanIntersection?bergmanRayIntersection(initialIdeal):bergmanRay(initialIdeal);
 		      
-		      //AsciiPrinter p(Stderr);
-		      //b.print(p);
-		  
 		      ret.codimensionOneCones.push_back(BergmanFan::CodimensionOneCone(initialIdeal));
-		      /*		  {
-					  AsciiPrinter p(Stderr);
-					  fprintf(Stderr,"Subfan:\n");
-					  b.print(p);
-					  }*/
+
+		      assert(b.cones.size());
+
+
 		      for(BergmanFan::MaximalConeList::const_iterator i=b.cones.begin();i!=b.cones.end();i++)
 			{
-			  assert(i->idealGroebnerBasis.isMarked());
+			  /* Unfortunately we have to make a lift to
+			     compute the relative interior point. This
+			     was not needed in earlier versions where
+			     symmetry checks were algebraic and not
+			     geometric*/
+			  //  assert(i->idealGroebnerBasis.isMarked());
+			  log2 cerr<<"8";
+
+			  PolynomialSet g2(theRing);
+			  WeightTermOrder termOrder(termorderWeight(initialIdeal));
+
+			  for(PolynomialSet::const_iterator j=i->idealGroebnerBasis.begin();j!=i->idealGroebnerBasis.end();j++)
+			    g2.push_back(divisionLift(*j, initialIdeal, current.idealGroebnerBasis, termOrder));
+			  assert(g2.isMarked());
+			  log2 cerr<<"9";
+			
+			 
+			  if(1)
+			    {
+			      log2 cerr << "AUTOREDUCTION";
+			      autoReduce(&g2,LexicographicTermOrder());
+			      log2 cerr << "AUTOREDUCTION END"<<endl;
+			    }
+			  else
+			    {
+			      log0 cerr << "RESTRICTED AUTOREDUCTION";
+			      autoReduceRestricted(&g2,i->idealGroebnerBasis);
+			      log0 cerr << "RESTRICTED AUTOREDUCTION END"<<endl;
+			    }
+			  ConeOrbit tempConeOrbit(*symmetryGroup,i->coneGroebnerBasis,g2,-1);
+			  log2 cerr<<"0";
+			  IntegerVector relIntFacetVector=tempConeOrbit.getStableRay();
+
+			  log2 cerr<<"A";
+
 			  int label=-1;
 			  IntegerVector labelPermutation;
-			  if(!active.containsAndMark(i->coneGroebnerBasis,initialIdeal,&label,&labelPermutation))
+			  if(!active.containsAndMark(relIntFacetVector,relIntRidgeVector,&label,&labelPermutation))
+			    //			  if(!active.containsAndMark(i->coneGroebnerBasis,initialIdeal,&label,&labelPermutation))
 			    {
+			  log2 cerr<<"B";
 			      if(!ret.contains(i->coneGroebnerBasis))
 				{
-				  /*fprintf(Stderr,"Lifting...\n");
-				    AsciiPrinter(Stderr).printPolynomialSet(i->idealGroebnerBasis);
-				  */
-				  PolynomialSet g2(theRing);
-				  for(PolynomialSet::const_iterator j=i->idealGroebnerBasis.begin();j!=i->idealGroebnerBasis.end();j++)
-				    g2.push_back(divisionLift(*j, initialIdeal, current.idealGroebnerBasis, LexicographicTermOrder()));
-				  //fprintf(Stderr,"Done lifting.\n");
-				  
-				  assert(g2.isMarked());
-				  //fprintf(Stderr,"Autoreducing...\n");
-				  
-				  //AsciiPrinter(Stderr).printPolynomialSet(g2);
-				  autoReduce(&g2,LexicographicTermOrder());
-				  //fprintf(Stderr,"Done autoreducing.\n");
-				  
-				  //  fprintf(Stderr,"Inserting:\n");
-				  //  AsciiPrinter(Stderr).printPolynomialSet(i->coneGroebnerBasis);
-				  //  AsciiPrinter(Stderr).printPolynomialSet(g2);
 				  label=maximalConeLabel++;
 				  labelPermutation=SymmetryGroup::identity(n);
 				  ConeOrbit newConeOrbit(*symmetryGroup,i->coneGroebnerBasis,g2,label);
 				  log1 fprintf(Stderr,"Adding orbit of size: %i\n",newConeOrbit.orbitSize());
-				  newConeOrbit.markFacet(initialIdeal);
+				  //newConeOrbit.markFacet(initialIdeal);
+				  newConeOrbit.markFacet(relIntRidgeVector);
 				  active.push_back(newConeOrbit);
 				}
+			      else
+				{
+				  assert(0);//Can this ever happen? REMOVE JAN 2009
+				}
+			  log2 cerr<<"C";
 			    }
+			  log2 cerr<<"D";
 			  ret.codimensionOneCones.back().incidenceList.push_back(label);
+			  log2 cerr<<"E";
 			  ret.codimensionOneCones.back().incidencePermutationList.push_back(labelPermutation);
+			  log2 cerr<<"F";
 			}
 		    }
-		  else
-		    numberOfAlreadyMarkedFacets++;
+		    //		  else
+		    //  numberOfAlreadyMarkedFacets++;
 		}
+
+	      else
+		{
+		  assert(0);
+		}
+
+
+
+
+
+
 	    }
-	  equalitySet[I]=0;
-	  i++;
+	    //	  equalitySet[I]=0;
+	    //  i++;
+
+	    //assert(0);//for timing purposes
 	}
       log1 fprintf(Stderr,"Done processing this orbit - Number of valid facets: %i   Number of already marked facets: %i\n",numberOfValidFacets,numberOfAlreadyMarkedFacets);
-      ret.cones.push_back(BergmanFan::MaximalCone(current.coneGroebnerBasis,current.idealGroebnerBasis,current.label,numberOfValidFacets));
+      ret.cones.push_back(BergmanFan::MaximalCone(current.coneGroebnerBasis,current.idealGroebnerBasis,false,current.label,numberOfValidFacets));
       active.pop_front();
     }
   ret.setSimplicial(isSimplicial);
@@ -565,11 +722,31 @@ BergmanFan bergman(PolynomialSet const &coneGroebnerBasis1, PolynomialSet const
 // BergmanFan
 //--------------------------------------
 
+BergmanFan::MaximalCone::MaximalCone(PolynomialSet const &coneGroebnerBasis_, PolynomialSet const &idealGroebnerBasis_, bool storeIdealBasis, int label_, int numberOfFacets_):
+  idealBasisStored(storeIdealBasis),
+  coneGroebnerBasis(coneGroebnerBasis_),
+  idealGroebnerBasis(idealGroebnerBasis_),
+  label(label_),
+  numberOfFacets(numberOfFacets_),
+  multiplicity(-1),
+  theCone(fastNormals(wallInequalities(idealGroebnerBasis_)), 
+	  wallInequalities(coneGroebnerBasis_),
+	  idealGroebnerBasis_.getRing().getNumberOfVariables())
+{
+  if(!storeIdealBasis)idealGroebnerBasis=PolynomialSet(coneGroebnerBasis_.getRing());
+  theCone.canonicalize();
+}
+
 int BergmanFan::numberOfMaximalCones()const
 {
   int ret=0;;
   for(MaximalConeList::const_iterator i=cones.begin();i!=cones.end();i++)
-    ret+=staticOrbitSize(i->coneGroebnerBasis, symmetryGroup);
+    {
+      IntegerVector stableRay=PolyhedralFan::stableRay(i->theCone,&symmetryGroup);
+      //ConeOrbit C(symmetryGroup, i->coneGroebnerBasis, i->idealGroebnerBasis, -1);
+      ret+=symmetryGroup.orbitSize(stableRay);//C.orbitSize();
+	//	ret+=staticOrbitSize(i->coneGroebnerBasis, symmetryGroup);
+    }
   return ret;
 }
 
@@ -588,10 +765,10 @@ void BergmanFan::print(Printer &p)
       p.printString("Orbit index: ");
       p.printInteger(i->label);
       p.printString("\n");
-      p.printString("Groebner pair:\n");
+      p.printString("Initial ideal:\n");
       p.printPolynomialSet(i->coneGroebnerBasis);
-      p.printPolynomialSet(i->idealGroebnerBasis);
-      int orbitSize=(symmetryGroup.sizeOfBaseSet())?staticOrbitSize(i->coneGroebnerBasis,symmetryGroup):1;
+      //p.printPolynomialSet(i->idealGroebnerBasis);
+      int orbitSize=(symmetryGroup.sizeOfBaseSet())?staticOrbitSize(i->coneGroebnerBasis,i->coneGroebnerBasis,symmetryGroup):1;
       p.printString("OrbitSize:");
       p.printInteger(orbitSize);
       p.printString("\n");
@@ -613,8 +790,13 @@ void BergmanFan::print(Printer &p)
       {
 	list<int> indices;
 	int index=0;
+
+
+	/*  //This needs to be changed so that torus permutations are considered too.
 	for(MaximalConeList::const_iterator j=cones.begin();j!=cones.end();j++,index++)
 	  if(staticInOrbit(j->coneGroebnerBasis,i->coneGroebnerBasis,symmetryGroup))indices.push_back(index);
+	*/
+
 	if(indices.size()>1)
 	  {
 	    fprintf(Stderr,"Conflicting orbits!!!!:");
@@ -713,8 +895,12 @@ PolyhedralFan BergmanFan::toPolyhedralFan()const
 
   for(MaximalConeList::const_iterator i=cones.begin();i!=cones.end();i++)
     {
-      PolyhedralCone c1(wallInequalities(i->idealGroebnerBasis),wallInequalities(i->coneGroebnerBasis));
+      //PolyhedralCone c1(wallInequalities(i->idealGroebnerBasis),wallInequalities(i->coneGroebnerBasis));
+      //      PolyhedralCone c1(wallFlipableNormals(i->idealGroebnerBasis,false),wallInequalities(i->coneGroebnerBasis),n);
+      PolyhedralCone c1=i->theCone;
+      log2 fprintf(Stderr,"Cononicalising...\n");
       c1.canonicalize();
+      log2 fprintf(Stderr,"... done canonicalising...\n");
       //      fprintf(Stderr,"a\n");
       //      for(SymmetryGroup::ElementContainer::const_iterator j=symmetryGroup.elements.begin();j!=symmetryGroup.elements.end();j++)
       //	{
