@@ -13,7 +13,7 @@ class FieldZModPZImplementation : public FieldImplementation
   const int p;
  public:
   FieldZModPZImplementation(int p_);
-  int getP();
+  int getP()const;
 };
 
 class FieldZModPZ : public Field
@@ -23,7 +23,7 @@ class FieldZModPZ : public Field
   int getP();
 };
 
-//Field *theZMod2ZField();
+Field *theZMod2ZField();
 
 
 
