@@ -1,5 +1,6 @@
 #include "halfopencone.h"
 
+#include <iostream>
 #include "buchberger.h"
 #include "enumeration.h"
 #include "reversesearch.h"
@@ -13,6 +14,65 @@
 #include "lp.h"
 #include "log.h"
 
+static void saveList(IntegerVectorList const &l, char const *name)
+{
+  FILE *f=fopen(name,"w");
+  if(f)
+    {
+      AsciiPrinter P(f);
+      P<<l;
+      fclose(f);
+    }
+}
+
+class HalfOpenConeProcessorVoid : public HalfOpenConeProcessor
+{
+public:
+  void process(HalfOpenCone const &c)
+  {
+  }
+};
+
+/* This Cone processor saves memory when doing up to symmetry computations where orbits can be repeated */
+class HalfOpenConeProcessorPointCollector : public HalfOpenConeProcessor
+{
+  int counter;
+public:
+  IntegerVectorList interiorPoints;
+  HalfOpenConeProcessorPointCollector():
+    counter(0)
+  {
+  }
+  void process(HalfOpenCone const &c)
+  {
+    HalfOpenCone c1=c;
+    PolyhedralCone c2=c1.closure();
+    c2.canonicalize();
+    interiorPoints.push_back(c2.getRelativeInteriorPoint());
+
+    if(savePartialResult)
+      {
+        assert(0);
+	counter++;
+	if(counter>=1000)
+	  saveList(interiorPoints,"partialresult");
+	counter=0;
+      }
+  }
+};
+
+/* This Cone processor saves memory when doing up to symmetry computations where orbits can be repeated */
+class HalfOpenConeProcessorConeCollector : public HalfOpenConeProcessor
+{
+public:
+  HalfOpenConeList theList;
+  void process(HalfOpenCone const &c)
+  {
+    theList.push_back(c);
+  }
+};
+
+
 static void printHalfOpenCone(Printer &P, HalfOpenCone c)
 {
   P.printPolyhedralCone(c.closure());
@@ -34,7 +94,7 @@ bool HalfOpenCone::contains(IntegerVector const &v)const
   IntegerVectorList equationList=lifted.getEquations();
 
   IntegerVectorList strict,nonstrict;
-  
+
 
   for(IntegerVectorList::const_iterator i=inequalityList.begin();i!=inequalityList.end();i++)
     if((*i)[i->size()-1]<0)
@@ -67,7 +127,7 @@ void HalfOpenCone::appendList(IntegerVectorList &to, IntegerVectorList const &fr
       v.resize(v.size()+1);
       v[v.size()-1]=appendValue;
       to.push_back(v);
-    }  
+    }
 }
 
 
@@ -80,7 +140,7 @@ HalfOpenCone::HalfOpenCone(int dimension_, PolyhedralCone const &lifted_):
 }
 
 
-HalfOpenCone::HalfOpenCone(int dimension_, IntegerVectorList const &equations, IntegerVectorList const &nonstrict, IntegerVectorList const &strict, bool findFacets):
+HalfOpenCone::HalfOpenCone(int dimension_, IntegerVectorList const &equations, IntegerVectorList const &nonstrict, IntegerVectorList const &strict, bool findFacets, bool canonicalize):
   dimension(dimension_),
   liftedDimension(dimension_+1),
   lifted(dimension_+1)
@@ -96,6 +156,7 @@ HalfOpenCone::HalfOpenCone(int dimension_, IntegerVectorList const &equations, I
   //      AsciiPrinter(Stderr).printInteger(liftedDimension);
   lifted=PolyhedralCone(inequalityList,equationList,liftedDimension);
   if(findFacets)lifted.findFacets();
+  if(canonicalize)lifted.canonicalize();
 }
 
 
@@ -129,6 +190,18 @@ HalfOpenCone::HalfOpenCone(PolyhedralCone C, TermOrder const &t):
 }
 
 
+HalfOpenCone::HalfOpenCone(int ambientDimension):
+  dimension(ambientDimension),
+  liftedDimension(ambientDimension+1),
+  lifted(ambientDimension+1)
+{
+  IntegerVectorList inequalityList;
+  inequalityList.push_back(IntegerVector::standardVector(liftedDimension,dimension));
+  IntegerVectorList empty;
+  lifted=PolyhedralCone(inequalityList,empty,liftedDimension);
+}
+
+
 static IntegerVectorList swapFirstLast(const IntegerVectorList &l)
 {
   IntegerVectorList ret;
@@ -160,7 +233,7 @@ bool HalfOpenCone::isEmpty()
   //  AsciiPrinter(Stderr).printVector(v);
   bool ret2=(v[dimension]==0);
     */
-  
+
   /*  fprintf(Stderr,"Inequalities:\n");
   AsciiPrinter(Stderr).printVectorList(lifted.getHalfSpaces());
   fprintf(Stderr,"Equations:\n");
@@ -210,7 +283,7 @@ bool haveEmptyIntersection(const HalfOpenCone &a, const HalfOpenCone &b)
 {
   IntegerVector v(liftedDimension);
   v[dimension]=-1;
-  
+
   IntegerVectorList equationList,inequalityList;
   inequalityList.push_back(v);
   PolyhedralCone c(inequalityList,equationList,liftedDimension);
@@ -224,7 +297,7 @@ bool haveEmptyIntersection(const HalfOpenCone &a, const HalfOpenCone &b)
 HalfOpenCone intersection(const HalfOpenCone &a, const HalfOpenCone &b, bool findFacets)
 {
   assert(a.dimension==b.dimension);
-    
+
   /*  fprintf(Stderr,"-----------------------------------------------------------\n");
   fprintf(Stderr,"Intersecting:\n");
   AsciiPrinter P(Stderr);
@@ -232,6 +305,8 @@ HalfOpenCone intersection(const HalfOpenCone &a, const HalfOpenCone &b, bool fin
   printHalfOpenCone(P,b);
   */
 
+
+
   HalfOpenCone ret=HalfOpenCone(a.dimension,intersection(a.lifted,b.lifted));
 
   {
@@ -244,7 +319,7 @@ HalfOpenCone intersection(const HalfOpenCone &a, const HalfOpenCone &b, bool fin
     //7
   }
 
-  
+
   /*  fprintf(Stderr,"Result:\n");
   printHalfOpenCone(P,ret);
   fprintf(Stderr,"Is empty:%i\n",ret.isEmpty());
@@ -275,7 +350,7 @@ PolyhedralCone HalfOpenCone::closure()
 
 /*
  */
-HalfOpenConeList orientedBoundary(PolyhedralCone C, TermOrder const &t)
+HalfOpenConeList orientedBoundary(PolyhedralCone C, TermOrder const &t, HalfOpenCone *restrictingCone)
 {
   int dimension=C.ambientDimension();
   HalfOpenConeList ret;
@@ -292,7 +367,8 @@ HalfOpenConeList orientedBoundary(PolyhedralCone C, TermOrder const &t)
       else
 	nonStrictList.push_back(*i);
     }
-
+  //  log0 AsciiPrinter(Stderr).printVectorList(strictList);
+  //  log0 AsciiPrinter(Stderr).printVectorList(nonStrictList);
   // Let's make the non-strict inequalities strict one at a time and add a cone for each iteration
   while(!nonStrictList.empty())
     {
@@ -303,7 +379,12 @@ HalfOpenConeList orientedBoundary(PolyhedralCone C, TermOrder const &t)
 
       IntegerVectorList equationList;
       equationList.push_back(v);
-      ret.push_back(HalfOpenCone(dimension,equationList,nonStrictList,strictList,true));
+      {
+	HalfOpenCone c(dimension,equationList,nonStrictList,strictList,true);
+	if(restrictingCone)
+	  c=intersection(*restrictingCone,c,true);
+	if(!c.isEmpty())ret.push_back(c);
+      }
       strictList.push_back(v);
     }
   return ret;
@@ -340,26 +421,34 @@ HalfOpenConeList tropicalHyperSurface(Polynomial const &p1)
   //  AsciiPrinter P(Stderr);
   //  printHalfOpenConeList(P,ret);
 
-  return ret;  
+  return ret;
 }
 
 
-static void buildFanFromPolynomial(Polynomial const &p1, HalfOpenConeList *fullDimCones, HalfOpenConeList *coDimOneCones, IntegerVector *parentList)
+static void buildFanFromPolynomial(Polynomial const &p1, HalfOpenConeList *fullDimCones, HalfOpenConeList *coDimOneCones, IntegerVector *parentList, HalfOpenCone *restrictingCone=0)
 {
+
   int n(p1.getRing().getNumberOfVariables());
 
+  IntegerVectorList empty;
+  HalfOpenCone dummy(n,empty,empty,empty,true);
+  if(restrictingCone==0)restrictingCone=&dummy;
+
   if(p1.isZero()) //If the polynomial is zero it makes most sense to return a complete fan consisting of one cone in both cases
     {
       IntegerVectorList empty;
-      if(fullDimCones)fullDimCones->push_back(HalfOpenCone(n,empty,empty,empty));
-      if(coDimOneCones)coDimOneCones->push_back(HalfOpenCone(n,empty,empty,empty));
+      if(fullDimCones)fullDimCones->push_back(intersection(HalfOpenCone(n,empty,empty,empty),*restrictingCone));
+      if(coDimOneCones)coDimOneCones->push_back(intersection(HalfOpenCone(n,empty,empty,empty),*restrictingCone));
       return;
     }
 
   LexicographicTermOrder myTermOrder;
   IntegerVectorList l=newtonPolytope(p1);
+
+  //  log0 AsciiPrinter(Stderr).printVectorList(l);
   assert(!l.empty());
   removeNonExtremeVerticesOfPolytope(l);
+  //  log0 AsciiPrinter(Stderr).printVectorList(l);
   IntegerVector parents;
   int numberOfCoDimOneCones=0;
   int numberOfFullDimCones=0;
@@ -370,15 +459,23 @@ static void buildFanFromPolynomial(Polynomial const &p1, HalfOpenConeList *fullD
       for(IntegerVectorList::const_iterator j=l.begin();j!=l.end();j++)
 	if(j!=i)
 	  inequalities.push_back(*i-*j);
-      
+
+      //      log0 fprintf(Stderr,"ineq\n");
+      //  log0 AsciiPrinter(Stderr).printVectorList(inequalities);
       inequalities=normalizedWithSumsAndDuplicatesRemoved(inequalities);
+      //log0 fprintf(Stderr,"ineq\n");
+      //log0 AsciiPrinter(Stderr).printVectorList(inequalities);
+
       IntegerVectorList empty;
-      PolyhedralCone C(inequalities,empty,n);
+      PolyhedralCone C(inequalities,empty,n,PCP_impliedEquationsKnown);
+
+
       C.findFacets();
 
       if(coDimOneCones || parentList)
 	{
-	  HalfOpenConeList temp=orientedBoundary(C,myTermOrder);
+	  HalfOpenConeList temp=orientedBoundary(C,myTermOrder,restrictingCone);
+	  //	  fprintf(Stderr,"TESTSET:%i\n",temp.size());
 	  for(int i=0;i<temp.size();i++)
 	    {
 	      numberOfCoDimOneCones++;
@@ -400,7 +497,7 @@ static void buildFanFromPolynomial(Polynomial const &p1, HalfOpenConeList *fullD
 	}
       if(fullDimCones)
 	{
-	  fullDimCones->push_back(HalfOpenCone(C,myTermOrder));
+	  fullDimCones->push_back(intersection(HalfOpenCone(C,myTermOrder),*restrictingCone));
 	  /*	  fprintf(Stderr,"Came from-------------------------------------------------\n");
 	  AsciiPrinter P(Stderr);
 	  fullDimCones->back().print(P);
@@ -471,27 +568,77 @@ HalfOpenConeList tropicalHyperSurfaceIntersection2(int dimension, PolynomialSet
 }
 
 
-PolyhedralFan tropicalHyperSurfaceIntersectionClosed(int dimension, PolynomialSet const &g)
+void tropicalHyperSurfaceIntersectionWithProcessor(int dimension, PolynomialSet const &g, HalfOpenConeProcessor &myProcessor, PolyhedralCone *restrictingCone, bool expand, int intervalLow, int intervalHigh)
 {
-  HalfOpenConeList intersection=tropicalHyperSurfaceIntersection(dimension,g);
-
-  log1 fprintf(Stderr,"Halfopen intersection completed.\n");
-
-  //  AsciiPrinter P(Stderr);
-  //  printHalfOpenConeList(intersection, P);
-
-  PolyhedralFan ret(dimension);
-  for(HalfOpenConeList::iterator i=intersection.begin();i!=intersection.end();i++)
+  assert(expand || !restrictingCone);//Need to fix this if we don't want to expand. Add in a different HalfOpenConeProcessor.
+  //  HalfOpenConeList intersection;
+  if(restrictingCone)
     {
-      PolyhedralCone c=i->closure();
-      c.canonicalize();
-      ret.insert(c);
-    }
+      IntegerVectorList equations=restrictingCone->getEquations();
+      IntegerVectorList nonStrict=restrictingCone->getHalfSpaces();
+      IntegerVectorList strict;
 
-  return ret;
+      /****************************************************************
+       * REMOVE THE LINE BELOW!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+       * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+       */
+//      strict.push_back(IntegerVector::standardVector(g.getRing().getNumberOfVariables(),0));cerr<<"REMOVE THIS LINE\n";
+
+      HalfOpenCone restrictingCone2(dimension,equations,nonStrict,strict,true);
+
+      //intersection=
+      tropicalHyperSurfaceIntersectionInSubspace(dimension,g,&restrictingCone2,myProcessor,intervalLow,intervalHigh);//<<---here
+      //intersection=tropicalHyperSurfaceIntersection(dimension,g,&restrictingCone2);
+    }
+    else
+      {
+	HalfOpenCone rCone(dimension);
+	//intersection=
+	tropicalHyperSurfaceIntersectionInSubspace(dimension,g,&rCone,myProcessor,intervalLow,intervalHigh);//<<---here
+	/*    PolyhedralFan intersectionOld=tropicalHyperSurfaceIntersection(dimension,g);
+      AsciiPrinter P(Stderr);
+      intersection.print(&P);
+      intersectionOld.print(&P);
+	*/
+      }
+  log2 fprintf(Stderr,"Halfopen intersection completed.\n");
 }
 
+PolyhedralFan tropicalHyperSurfaceIntersectionClosed(int dimension, PolynomialSet const &g, PolyhedralCone *restrictingCone, bool expand, bool saveResult, int intervalLow, int intervalHigh)
+{
+	  HalfOpenConeProcessorPointCollector myProcessor;
+	  if(saveResult)myProcessor.setSave();
 
+	  tropicalHyperSurfaceIntersectionWithProcessor(dimension,g,myProcessor,restrictingCone,expand,intervalLow,intervalHigh);
+
+	  if(saveResult)saveList(myProcessor.interiorPoints,"finalresult");
+
+
+	  //  AsciiPrinter P(Stderr);
+	  //  printHalfOpenConeList(intersection, P);
+
+	  PolyhedralFan ret(dimension);
+	  //  for(HalfOpenConeList::iterator i=intersection.begin();i!=intersection.end();i++)
+	  for(IntegerVectorList::const_iterator i=myProcessor.interiorPoints.begin();i!=myProcessor.interiorPoints.end();i++)
+	    {
+	      /*PolyhedralCone c=i->closure();
+	      c.canonicalize();
+
+	      if(expand)
+		c=normalConeOfMinkowskiSum(g,c.getRelativeInteriorPoint());
+
+	      ret.insert(c);
+	      */
+	      ret.insert(normalConeOfMinkowskiSum(g,*i));
+	    }
+	  /*      AsciiPrinter P(Stderr);
+	      ret.print(&P);
+	      cerr<<"ETSTSETST------------------------------------------------------------------------------";
+	  */
+
+	  return ret;
+
+}
 void HalfOpenCone::splitIntoRelativelyOpenCones(list<HalfOpenCone> &l)
 {
   //  fprintf(Stderr,"BEGIN\n");
@@ -499,7 +646,7 @@ void HalfOpenCone::splitIntoRelativelyOpenCones(list<HalfOpenCone> &l)
   //  print(P);
   lifted.findFacets();
   //  print(P);
-  
+
   /*	{
 	  IntegerVector v=StringParser("(3,0,3,2,0,3)").parseIntegerVector();
 	  if(contains(v))fprintf(Stderr,"??????????????????????????????????????????\n");
@@ -509,7 +656,7 @@ void HalfOpenCone::splitIntoRelativelyOpenCones(list<HalfOpenCone> &l)
   IntegerVectorList equationList=lifted.getEquations();
 
   IntegerVectorList strict,nonstrict;
-  
+
 
   for(IntegerVectorList::const_iterator i=inequalityList.begin();i!=inequalityList.end();i++)
     if((*i)[i->size()-1]<0)
@@ -546,7 +693,7 @@ void HalfOpenCone::splitIntoRelativelyOpenCones(list<HalfOpenCone> &l)
 
       HalfOpenCone A(dimension,PolyhedralCone(a,equationList,liftedDimension));
       A.splitIntoRelativelyOpenCones(l);
-      
+
 
       equationList.push_front(chosen);
       strict.pop_front();
@@ -654,7 +801,7 @@ void printHalfOpenConeList(HalfOpenConeList const &l, class Printer & p)
   IntegerVectorList rays;
   for(list<PolyhedralCone>::iterator i=cones.begin();i!=cones.end();i++)
     if(i->dimension()==homog+1)rays.push_back(i->getRelativeInteriorPoint());
-  
+
   p.printString("Rays:\n");
 
   p.printVectorList(rays,true);
@@ -704,7 +851,7 @@ void printHalfOpenConeList(HalfOpenConeList const &l, class Printer & p)
 	}
       else
 	maximal=*subsetIterator;
-      
+
       p.printString("Printing ");p.printInteger(subsetIterator->size());p.printString(" ");p.printInteger(d);p.printString("-dimensional cones (");p.printInteger(maximal.size());p.printString(" maximal cones):\n");
       p.printString("{");
       {
@@ -809,8 +956,86 @@ public:
     for(int i=0;i<size();i++)if(v[i])ret++;
     return ret;
   }
-};
+  };
+/*
+class BitSet
+{
+#define BITSET_BITS_PER_WORD 32
+  vector<int> v;
+  int ambientSetSize;
+  static int bitsSetInInt(int i)
+  {
+    if(BITSET_BITS_PER_WORD==32)
+      {
+	i=(i&0x55555555)+((i>>1)&0x55555555);
+	i=(i&0x33333333)+((i>>2)&0x33333333);
+	i=(i&0x0f0f0f0f)+((i>>4)&0x0f0f0f0f);
+	i=(i)+((i>>8));
+	i=(i)+((i>>16));
+	return i&255;
+      }
 
+
+    int ret=0;
+    while(i&(i-1)){i&=i-1;ret++;}
+    return ret;
+  }
+public:
+  BitSet()
+  {
+  }
+  BitSet(int n):
+    ambientSetSize(n),
+    v(((unsigned int)(ambientSetSize-1))/BITSET_BITS_PER_WORD+1)
+  {
+    for(int i=0;i<v.size();i++)v[i]=0;
+  }
+  //  int& operator[](unsigned int n){assert(n>=0 && n<ambientSize);return (v[n/BITSET_BITS_PER_WORD]&&);}
+  inline bool operator[](unsigned int n)const{assert(n>=0 && n<ambientSetSize);return (v[n/BITSET_BITS_PER_WORD]&(((int)1)<<(n&(BITSET_BITS_PER_WORD-1))));}
+  inline void setValue(unsigned int index, bool value)
+  {
+    if(value)
+      v[index/BITSET_BITS_PER_WORD]|=((int)1)<<(index&(BITSET_BITS_PER_WORD-1));
+    else
+      v[index/BITSET_BITS_PER_WORD]&=-1-(((int)1)<<(index&(BITSET_BITS_PER_WORD-1)));
+  }
+  void add(BitSet const &b)
+  {
+    assert(b.size()==ambientSetSize);
+    for(int i=0;i<ambientSetSize;i++)v[i]|=b.v[i];
+  }
+  int size()const
+  {
+    return ambientSetSize;
+  }
+  void print(Printer &P)const
+  {
+    P.printString("(");
+    for(int i=0;i<ambientSetSize;i++)
+      {
+	if(i!=0)P.printString(", ");
+	P.printInteger((*this)[i]);
+      }
+    P.printString(")\n");
+  }
+  BitSet negated()const
+  {
+    BitSet ret(ambientSetSize);
+    for(int i=0;i<v.size();i++)ret.v[i]=-1-v[i];
+    return ret;
+  }
+  int sizeOfSubset()const
+  {
+    int ret=0;
+    for(int i=0;i<v.size();i++)ret+=bitsSetInInt(v[i]);
+    return ret;
+  }
+  int sizeOfComplement()const
+  {
+    return ambientSetSize-sizeOfSubset();
+  }
+};
+*/
 class Table
 {
   vector<vector<vector<BitSet> > > table;
@@ -850,6 +1075,8 @@ public:
 
     table[fan1][fan2][cone1][cone2]=true;
     table[fan2][fan1][cone2][cone1]=true;
+    //table[fan1][fan2][cone1].setValue(cone2,true);
+    //table[fan2][fan1][cone2].setValue(cone1,true);
   }
   BitSet const& nonCandidates(int fan1, int cone1, int fan2)const
   {
@@ -885,7 +1112,7 @@ public:
     knownNonEmptyIntersection(l),
     numberOfSolvedLPs(0)
   {
-    
+
   }
   bool knownToIntersectTriviallyInIntersection(int fan1, int cone1, int fan2, int cone2)
   {
@@ -920,7 +1147,7 @@ public:
   const BitSet &getNonCandidates(int fan1, int cone1, int fan2)
   {
     //    for(int c2=0;c2<fanList[fan2].size();c2++)
-    //      intersectTriviallyInIntersection(fan1,cone1,fan2,c2);    
+    //      intersectTriviallyInIntersection(fan1,cone1,fan2,c2);
 
     return knownEmptyIntersectionInIntersection.nonCandidates(fan1,cone1,fan2);
   }
@@ -928,6 +1155,10 @@ public:
   {
     knownEmptyIntersectionInIntersection.set(fan1,cone1,fan2,cone2);
   }
+  void markKnownNonEmptyIntersection(int fan1, int cone1, int fan2, int cone2)
+  {
+	  knownNonEmptyIntersection.set(fan1,cone1,fan2,cone2);
+  }
   void print()const
   {
     fprintf(Stderr,"knownEmptyIntersectionInIntersection:");
@@ -938,6 +1169,7 @@ public:
 };
 
 
+
 struct RecursionData
 {
   vector<vector<HalfOpenCone> > fans;
@@ -948,9 +1180,12 @@ struct RecursionData
   BitSet usedFans;
   int numberOfUsefulCalls;
   int totalNumberOfCalls;
+  HalfOpenConeProcessor &processor;
+  bool intervalSet;
+  int intervalLow,intervalHigh;
 public:
   RelationTable table;
-  RecursionData(vector<vector<HalfOpenCone> > const &fans_):
+  RecursionData(vector<vector<HalfOpenCone> > const &fans_, HalfOpenConeProcessor &processor_):
     table(fans_),
     fans(fans_),
     chosen(fans_.size()),
@@ -959,11 +1194,18 @@ public:
     iterators(fans_.size()),
     nCandidates(fans_.size()),
     numberOfUsefulCalls(0),
-    totalNumberOfCalls(0)
+    totalNumberOfCalls(0),
+    processor(processor_),
+    intervalSet(false)
   {
   }
-
-  HalfOpenConeList ret;
+  void setInterval(int lo, int hi)
+  {
+    intervalLow=lo;
+    intervalHigh=hi;
+    intervalSet=true;
+  }
+  //  HalfOpenConeList ret;
 
   bool randBool()const
   {
@@ -994,8 +1236,9 @@ public:
 	  if(table.intersectTriviallyInIntersection(chosenFans[i], chosen[i], fanNumber, j))
 	    {
 	      nonCandidates[j]=true;
+	      //	      nonCandidates.setValue(j,true);
 	      break;
-	    }	    
+	    }
 
     return nonCandidates.negated();
   }
@@ -1003,14 +1246,16 @@ public:
   {
     totalNumberOfCalls++;
 
-    if((totalNumberOfCalls&15) == 0)closure();
+    //    if((totalNumberOfCalls&15) == 0)closure();
+    //    if((totalNumberOfCalls&255) == 255)closure();
 
     bool success=false;
 
     if(index == fans.size())
       {
 	log2 fprintf(Stderr,"ADDING CONE\n");
-	ret.push_back(current);
+	//ret.push_back(current);
+	processor.process(current);
 	numberOfUsefulCalls++;
 	return true;
       }
@@ -1039,6 +1284,7 @@ public:
 
 	chosenFans[index]=bestIndex;
 	usedFans[chosenFans[index]]=true;
+	//usedFans.setValue(chosenFans[index],true);
 
 
 	nCandidates[index]=bestNumberOfCandidates;//just for printing
@@ -1090,6 +1336,14 @@ public:
 	for(int i=0;i<fans[chosenFans[index]].size();i++)
 	  if(candidates[i])
 	    {
+	      if(intervalSet && (index==0))
+		{
+		  if(i<intervalLow || i>=intervalHigh)
+		    {
+		      cerr << "SKIPPING CONE "<<i<<endl;
+		      continue;
+		    }
+		}
 	      bool ok=true;
 	      for(int j=0;j<index;j++)
 		{
@@ -1097,19 +1351,22 @@ public:
 		    {
 		      ok=false;
 		      break;
+
 		    }
 		}
 	      if(ok && !haveEmptyIntersection(current,fans[chosenFans[index]][i]))
 		{
 		  chosen[index]=i;
 
+		  //log0 fprintf(Stderr,"A\n");
 		  HalfOpenCone next=intersection(current,fans[chosenFans[index]][i],true);
-		  if((fans.size()!=10))//&&(fans.size()!=9))
+		  //log0 fprintf(Stderr,"B\n");
+		  //if((fans.size()!=10))//&&(fans.size()!=9))
 		    {
 		      bool s=rek(index+1,next);
 		      success|=s;
 		    }
-		  else
+		  /*		  else //Some old optimization code if we have 10 fans????
 		    {
 		      vector<vector<HalfOpenCone> > L2;
 		      vector<int> indicesOfNonUsedFans;
@@ -1128,7 +1385,7 @@ public:
 				      indicesOfCones.back().push_back(j);
 				    }
 				}
-			      fprintf(Stderr,"New fan size:%i\n",L.size()); 
+			      fprintf(Stderr,"New fan size:%i\n",L.size());
 			      L2.push_back(L);
 			      indicesOfNonUsedFans.push_back(i);
 			    }
@@ -1148,7 +1405,7 @@ public:
 		      IntegerVectorList empty;
 		      success|=data.rek(0, HalfOpenCone(next.dimension,empty,empty,empty));
 		      ret.splice(ret.begin(),data.ret);
-		    }
+		    }*/
 		  chosen[index]=-1;//just for printing
 		}
 	      iterators[index]++;//just for printing
@@ -1173,6 +1430,7 @@ public:
 	iterators[index]=0;//just for printing
 
 	usedFans[chosenFans[index]]=false;
+	//	usedFans.setValue(chosenFans[index],false);
 	chosenFans[index]=-1;
       }
     if(success)numberOfUsefulCalls++;
@@ -1180,6 +1438,7 @@ public:
   }
   bool closure()
   {
+    log2 cerr<<"computing closure"<<endl;
     bool ret=false;
     int a=0;
     for(int f1=0;f1<fans.size();f1++)
@@ -1222,6 +1481,7 @@ public:
 	      }
       }
     log2 fprintf(Stderr,"%i FOR FREE\n",a);
+    log2 cerr<<"done computing closure"<<endl;
     return ret;
   }
 
@@ -1232,23 +1492,34 @@ public:
 
   void completeTable()
   {
-    for(int f1=0;f1<fans.size();f1++)
+	    for(int f1=0;f1<fans.size();f1++)
+	    	for(int c1=0;c1<fans[f1].size();c1++)
+	    	  for(int c2=0;c2<fans[f1].size();c2++)
+	    		  if(c1!=c2)
+	    			  table.markNoIntersectionInIntersection(f1,c1,f1,c2);
+//	    		  else
+//	    			  table.markKnownNonEmptyIntersection(f1,c1,f1,c2);
+
+
+	    	for(int f1=0;f1<fans.size();f1++)
       {
 	log3 fprintf(Stderr,"%i\n",f1);
 	for(int f2=f1+1;f2<fans.size();f2++)
 	for(int c1=0;c1<fans[f1].size();c1++)
 	  for(int c2=0;c2<fans[f2].size();c2++)
-	    table.intersectTriviallyInIntersection(f1,c1,f2,c2);    
+	    table.intersectTriviallyInIntersection(f1,c1,f2,c2);
       }
     transitiveClosure();
+//    table.print();//HERE
   }
   void extractInformationFromFullFan(vector<vector<HalfOpenCone> > const &fans2, vector<IntegerVector> parentList)
   {
     int freeLPs=0;
-    RecursionData data2(fans2);
+    HalfOpenConeProcessorVoid dummy;
+    RecursionData data2(fans2,dummy);
     data2.completeTable();
     data2.transitiveClosure();
-    
+
     assert(fans.size()==fans2.size());
     assert(fans.size()==parentList.size());
     //    data2.table.print();
@@ -1277,7 +1548,7 @@ public:
 		    A2.print(P);
 		    B2.print(P);
 		    }*/
-		
+
 
 		//		assert(table.intersectTriviallyInIntersection(f1,c1,f2,c2));
 		table.markNoIntersectionInIntersection(f1, c1, f2, c2);
@@ -1289,12 +1560,14 @@ public:
 };
 
 
-HalfOpenConeList tropicalHyperSurfaceIntersection(int dimension, PolynomialSet const &g)
+
+void tropicalHyperSurfaceIntersection(int dimension, PolynomialSet const &g, HalfOpenCone *restrictingCone, HalfOpenConeProcessor &processor)
 {
   vector<vector<HalfOpenCone> > fullDimFanList;
   vector<vector<HalfOpenCone> > coDimOneFanList;
   vector<IntegerVector> parents(g.size());
 
+  //  AsciiPrinter(Stderr) << g;
 
   //  fprintf(Stderr,"GSIZE:%i\n",g.size());
   {
@@ -1302,7 +1575,7 @@ HalfOpenConeList tropicalHyperSurfaceIntersection(int dimension, PolynomialSet c
     for(PolynomialSet::const_iterator i=g.begin();i!=g.end();i++)
       {
 	HalfOpenConeList l,lf;
-	buildFanFromPolynomial(*i, &lf, &l, &(parents[I]));
+	buildFanFromPolynomial(*i, &lf, &l, &(parents[I]),restrictingCone);
 
 	log2 AsciiPrinter(Stderr).printVector(parents[I]);
 
@@ -1325,17 +1598,348 @@ HalfOpenConeList tropicalHyperSurfaceIntersection(int dimension, PolynomialSet c
       }
   }
 
-  RecursionData data(coDimOneFanList);
+  RecursionData data(coDimOneFanList,processor);
   data.extractInformationFromFullFan(fullDimFanList,parents);
-  //  data.completeTable();
-  //  data.table.print();
+
+//  data.completeTable();//HERE
+//  data.table.print();//HERE
   IntegerVectorList empty;
   data.rek(0, HalfOpenCone(dimension,empty,empty,empty));
   log2 fprintf(Stderr,"LPs solved:%i for relation table\n",data.table.numberOfSolvedLPs);
-  return data.ret;
 }
 
 
+HalfOpenConeList tropicalHyperSurfaceIntersection(int dimension, PolynomialSet const &g, HalfOpenCone *restrictingCone)
+{
+  HalfOpenConeProcessorConeCollector collector;
+  tropicalHyperSurfaceIntersection(dimension, g, restrictingCone, collector);
+  return collector.theList;
+}
+
+
+#include "binomial.h"
+#include "linalg.h"
+
+//ignore last coordinate. Get rid of pivots
+static IntegerVectorList rewriteVectorList(IntegerVectorList const &l, list<int> nonPivots, FieldMatrix const &A)
+{
+  IntegerVectorList ret;
+  int M=nonPivots.size();
+
+  for(IntegerVectorList::const_iterator i=l.begin();i!=l.end();i++)
+    {
+      IntegerVector v(M+1);
+      v[M]=(*i)[i->size()-1];
+      IntegerVector u=A.canonicalize(integerVectorToFieldVector(i->subvector(0,i->size()-1),Q)).primitive();
+      int J=0;
+      for(list<int>::const_iterator j=nonPivots.begin();j!=nonPivots.end();j++,J++)v[J]=u[*j];
+      ret.push_back(v);
+    }
+
+  return ret;
+}
+
+//add pivot columns
+static IntegerVectorList expandVectorList(IntegerVectorList const &l, list<int> pivots)
+{
+  IntegerVectorList ret;
+  int pivotSize=pivots.size();
+
+  for(IntegerVectorList::const_iterator i=l.begin();i!=l.end();i++)
+    {
+      int newSize=pivotSize+i->size();
+
+      IntegerVector v(newSize);
+
+      list<int>::const_iterator J1=pivots.begin();
+      int J2=0;
+      for(int j=0;j<newSize;j++)
+	{
+	  if((J1!=pivots.end())&&(((*J1))==j))
+	    {
+	      v[j]=0;
+	      J1++;
+	    }
+	  else
+	    {
+	      v[j]=(*i)[J2];
+	      J2++;
+	    }
+	}
+
+      ret.push_back(v);
+    }
+
+  return ret;
+}
+
+
+HalfOpenCone HalfOpenCone::withChosenCoordinates(list<int> chosen)const
+{
+  chosen.push_back(dimension);
+  PolyhedralCone newLifted=PolyhedralCone(
+					  subvectorsOfIntegerVectorList(lifted.getHalfSpaces(),chosen),
+					  subvectorsOfIntegerVectorList(lifted.getEquations(),chosen),
+					  chosen.size()
+					  );
+  return HalfOpenCone(chosen.size()-1,newLifted);
+}
+
+
+HalfOpenCone HalfOpenCone::rewrite(FieldMatrix const &A, list<int> nonPivots)const
+{
+  PolyhedralCone newLifted=PolyhedralCone(
+					  rewriteVectorList(lifted.getHalfSpaces(),nonPivots,A),
+					  rewriteVectorList(lifted.getEquations(),nonPivots,A),
+					  nonPivots.size()+1
+					  );
+  return HalfOpenCone(nonPivots.size(),newLifted);
+}
+
+
+HalfOpenCone HalfOpenCone::rewriteExpand(list<int> pivots, IntegerVectorList const &newEquations)const
+{
+  IntegerVectorList equations=expandVectorList(lifted.getEquations(),pivots);
+
+  for(IntegerVectorList::const_iterator i=newEquations.begin();i!=newEquations.end();i++)
+    {
+      IntegerVector v(i->size()+1);
+      for(int j=0;j<v.size()-1;j++)v[j]=(*i)[j];
+      equations.push_back(v);
+    }
+
+  PolyhedralCone newLifted=PolyhedralCone(
+					  expandVectorList(lifted.getHalfSpaces(),pivots),
+					  equations,
+					  dimension+pivots.size()+1
+					  );
+  return HalfOpenCone(dimension+pivots.size(),newLifted);
+}
+
+static void print(HalfOpenConeList const &l, Printer &P)
+{
+  P << "Printing half Open cone list-----------------------------------------------------------------\n";
+  for(HalfOpenConeList::const_iterator i=l.begin();i!=l.end();i++)
+    {
+      i->lifted.print(&P);
+    }
+  P << "Done printing half Open cone list-----------------------------------------------------------------\n";
+}
+static void print2(vector<HalfOpenCone> const &l, Printer &P)
+{
+  P << "Printing half Open cone list---------------------------------------------------------------\n";
+  for(vector<HalfOpenCone>::const_iterator i=l.begin();i!=l.end();i++)
+    {
+      i->lifted.print(&P);
+    }
+  P << "Done printing half Open cone list----------------------------------------------------------------!\n";
+}
+
+static IntegerVectorList liftEquations(IntegerVectorList const &l, FieldMatrix const &B)
+{
+  IntegerVectorList ret;
+  list<int> nonPivots=B.nonPivotColumns();
+
+  for(IntegerVectorList::const_iterator i=l.begin();i!=l.end();i++)
+    {
+      FieldVector I=integerVectorToFieldVector(i->subvector(0,i->size()-1),Q);
+      IntegerVector v(B.getWidth()+1);
+      FieldVector v2(Q,B.getWidth());
+      for(int j=0;j<B.getWidth();j++)
+	v2[j]=dot(B.canonicalize(integerVectorToFieldVector(IntegerVector::standardVector(B.getWidth(),j),Q)).subvector(nonPivots),I);
+      IntegerVector v3=v2.primitive();
+      for(int j=0;j<v3.size();j++)v[j]=v3[j];
+      v[B.getWidth()]=(*i)[i->size()-1];
+      ret.push_back(v);
+    }
+
+  return ret;
+}
+
+HalfOpenCone liftEquations(HalfOpenCone const &c, FieldMatrix const &B)
+{
+  PolyhedralCone newCone=PolyhedralCone(
+					  liftEquations(c.lifted.getHalfSpaces(),B),
+					  liftEquations(c.lifted.getEquations(),B),
+					  B.getWidth()+1
+					  );
+  return HalfOpenCone(B.getWidth(),newCone);
+}
+
+/* Use full for transforming the cone back into original coordinates*/
+class HalfOpenConeProcessorAffineChange : public HalfOpenConeProcessor
+{
+  IntegerVectorList interiorPoints;
+  HalfOpenConeProcessor &parent;
+  list<int> pivots;
+  IntegerVectorList newEquations;
+  FieldMatrix B;
+public:
+  HalfOpenConeProcessorAffineChange(HalfOpenConeProcessor &parent_, list<int> const &pivots_, IntegerVectorList const &newEquations_, FieldMatrix const &B_):
+    parent(parent_),
+    pivots(pivots_),
+    newEquations(newEquations_),
+    B(B_)
+  {
+    if(parent_.savePartialResult)setSave();
+  }
+  void process(HalfOpenCone const &c)
+  {
+    HalfOpenCone c2=liftEquations(c.rewriteExpand(pivots,newEquations),B);
+    parent.process(c2);
+  }
+};
+
+
+//HalfOpenConeList
+void tropicalHyperSurfaceIntersectionInSubspace(int dimension, PolynomialSet const &G, HalfOpenCone *restrictingCone, HalfOpenConeProcessor &processor, int intervalLow, int intervalHigh)
+{
+  /*
+    Removing the lineality space. (This could be carried out on the
+    level of polyhedra, which may or may not be more efficient.)
+
+    We first find generators for the linalityspace, write them as rows
+    of a matrix B.  We reduce B and wish to ignore the coordinates
+    whose columns contains a pivot.
+
+    Afterwards we need to expand the inequalities/equations to the
+    entire space. The matrix B tels us how to project to
+    lowerdimensional space (by taking normals forms). The equations
+    are gotten as the pull-back of the lower dimensional equations by
+    this normal form map.
+   */
+  log2 cerr<<"Projecting Newton polytopes modulo the homogeneity space.";
+
+  int N=G.getRing().getNumberOfVariables();
+  IntegerVectorList w=wallInequalities(G);
+  FieldMatrix W=integerMatrixToFieldMatrix(rowsToIntegerMatrix(w,N),Q);
+  if(restrictingCone)
+    {
+      PolyhedralCone temp=restrictingCone->closure();
+      W=combineOnTop(combineOnTop(W,
+					  integerMatrixToFieldMatrix(rowsToIntegerMatrix(temp.getEquations(),N),Q)),
+			                  integerMatrixToFieldMatrix(rowsToIntegerMatrix(temp.getHalfSpaces(),N),Q));
+    }
+  FieldMatrix B=W.reduceAndComputeKernel();
+  B.reduce();
+  list<int> BNonPivots=B.nonPivotColumns();
+  PolynomialRing R(G.getRing().getField(),BNonPivots.size());
+
+  PolynomialSet g=G.embeddedInto(R,&BNonPivots);
+
+  HalfOpenCone restrictedConeNew(0,PolyhedralCone(1));
+  if(restrictingCone)
+    {
+      restrictedConeNew=restrictingCone->withChosenCoordinates(BNonPivots);
+      restrictingCone=&restrictedConeNew;
+    }
+
+  log2 cerr<<"Done projecting Newton polytopes modulo the homogeneity space.";
+  /*
+    Now do the computation with the new set of polynomials.
+   */
+
+  /*
+    Here follows restriction to subspace cut out by the binomials.
+   */
+  log2 cerr<<"Restricting to subspace determined by binomials and computing tropical hypersurfaces.";
+
+  int n=g.getRing().getNumberOfVariables();
+  IntegerVectorList equations;
+
+  if(restrictingCone)//add those from the restricting cone
+    {
+      equations=restrictingCone->shrink(restrictingCone->lifted.getEquations());
+    }
+
+  int numberOfNonBinomials=0;
+  for(PolynomialSet::const_iterator i=g.begin();i!=g.end();i++)
+    {
+      if(i->numberOfTerms()==2)
+	equations.push_back(binomialToIntegerVector(*i));
+      else
+	numberOfNonBinomials++;
+    }
+  FieldMatrix A=integerMatrixToFieldMatrix(rowsToIntegerMatrix(equations,n),Q);
+
+  int npivots=A.reduceAndComputeRank();
+  // Coordinates to remove are those corresponding to pivots
+  // We want to rewrite inequalities without the non-pivots
+  // Unfortunatetly we need to compute the fans first
+
+
+  //  AsciiPrinter(Stderr)<<g;
+
+  list<int> nonPivots=A.nonPivotColumns();
+
+  //  log0 cerr<<"Coordinates left "<<nonPivots.size()<<endl;
+
+  vector<vector<HalfOpenCone> > fullDimFanList;
+  vector<vector<HalfOpenCone> > coDimOneFanList;
+  vector<IntegerVector> parents(numberOfNonBinomials);
+
+    list<int> pivots=A.pivotColumns();
+
+  int I=0;
+  for(PolynomialSet::const_iterator i=g.begin();i!=g.end();i++)
+    if(i->numberOfTerms()!=2)
+      {
+	log2 cerr << I;
+	HalfOpenConeList l,lf;
+	log2 cerr<< "Building fan"<<endl;
+	buildFanFromPolynomial(*i, &lf, &l, &(parents[I]),restrictingCone);
+	log2 cerr<< "Number of cones:" << lf.size()<<","<< l.size()<<endl;
+	log2 cerr<< "rewriting"<<endl;
+
+	vector<HalfOpenCone> L;
+	for(HalfOpenConeList::const_iterator i=l.begin();i!=l.end();i++)
+	  L.push_back(i->rewrite(A,nonPivots));
+	vector<HalfOpenCone> F;
+	for(HalfOpenConeList::const_iterator i=lf.begin();i!=lf.end();i++)
+	  F.push_back(i->rewrite(A,nonPivots));
+
+	fullDimFanList.push_back(F);
+	coDimOneFanList.push_back(L);
+	log2 cerr<< "Done Building fan"<<endl;
+	I++;
+      }
+  log2 cerr<<"Done computing tropical hypersurfaces.";
+
+  /* Now we must create a new HalfOpenConeProcessor and tell it how to expand a cone.
+     We must insert the pivot columns from A and add in the equtions gotten from the binomials.
+   */
+  A.removeZeroRows();
+  IntegerVectorList newEquations=fieldMatrixToIntegerMatrixPrimitive(A).getRows();
+
+  HalfOpenConeProcessorAffineChange myProcessor(processor,pivots,newEquations,B);
+
+  RecursionData data(coDimOneFanList,myProcessor);
+  data.extractInformationFromFullFan(fullDimFanList,parents);
+
+//  data.completeTable();//HERE ENABLE HERE
+//  data.table.print();//HERE
+
+
+  IntegerVectorList empty;
+  log2 cerr<<"Doing intersection.";
+  if(intervalHigh!=-1)data.setInterval(intervalLow,intervalHigh);
+  data.rek(0, HalfOpenCone(nonPivots.size(),empty,empty,empty));
+  log2 cerr<<"Done doing intersection.";
+  //  data.rek(0, HalfOpenCone(dimension,empty,empty,empty));
+  //  log2 fprintf(Stderr,"LPs solved:%i for relation table\n",data.table.numberOfSolvedLPs);
+
+  /*  HalfOpenConeList ret;
+
+  for(HalfOpenConeList::const_iterator i=data.ret.begin();i!=data.ret.end();i++)
+    {
+      //      ret.push_back(i->rewriteExpand(pivots,newEquations));
+      ret.push_back(liftEquations(i->rewriteExpand(pivots,newEquations),B));
+    }
+
+  return ret;
+  */
+}
+
 
 PolyhedralFan faceComplexOfCone(HalfOpenCone &c)
 {
@@ -1350,3 +1954,80 @@ PolyhedralFan faceComplexOfCone(HalfOpenCone &c)
     }
   return ret;
 }
+
+
+bool operator<(HalfOpenCone const &a, HalfOpenCone const &b)
+{
+  return a.lifted<b.lifted;
+}
+
+
+class SingleSubspaceProcessor : public HalfOpenConeProcessor
+{
+  int desiredDimension;
+  bool desiredDimensionFound;
+public:
+  SingleSubspaceProcessor(int desiredDimension_):
+    desiredDimension(desiredDimension_),
+    desiredDimensionFound(false)
+  {
+  }
+  void process(HalfOpenCone const &c)
+  {
+    if(c.lifted.dimension()-1==desiredDimension)
+      {
+        desiredDimensionFound=true;
+//        return false;
+      }
+//    return true;
+  }
+  bool success()
+  {
+    return desiredDimensionFound;
+  }
+};
+
+bool nonEmptyStableIntersection(PolynomialSet const &g)
+{
+  vector<vector<HalfOpenCone> > coDimOneFanList;
+
+  {
+    int I=0;
+    for(PolynomialSet::const_iterator i=g.begin();i!=g.end();i++)
+      {
+        HalfOpenConeList l;
+        PolynomialSet A(g.getRing());
+        A.push_back(*i);
+        A.markAndScale(LexicographicTermOrder());
+        PolyhedralCone h=homogeneitySpace(A);
+        h.canonicalize();
+        IntegerVectorList equations=h.getEquations();
+        for(IntegerVectorList::const_iterator j=equations.begin();j!=equations.end();j++)
+          {
+            IntegerVectorList eq2;
+            eq2.push_back(*j);
+            HalfOpenCone C(j->size(),eq2,IntegerVectorList(),IntegerVectorList());
+            l.push_back(C);
+          }
+
+
+        vector<HalfOpenCone> L;
+        for(HalfOpenConeList::const_iterator i=l.begin();i!=l.end();i++)
+          {
+            L.push_back(*i);
+          }
+        coDimOneFanList.push_back(L);
+        I++;
+      }
+  }
+
+  SingleSubspaceProcessor p(g.getRing().getNumberOfVariables()-g.size());
+  RecursionData data(coDimOneFanList,p);
+
+  IntegerVectorList empty;
+  data.rek(0, HalfOpenCone(g.getRing().getNumberOfVariables(),empty,empty,empty));
+
+  log2 fprintf(Stderr,"LPs solved:%i for relation table\n",data.table.numberOfSolvedLPs);
+
+  return p.success();
+}
