@@ -4,12 +4,13 @@
 #include "cdd_f.h"
 #include "termorder.h"
 #include "printer.h"
+#include "log.h"
 
 //--------------------------------------------------
 // LpSolverCdd (double precision)
 //--------------------------------------------------
 
-static ddf_MatrixPtr vectorList2Matrix(const IntegerVectorList &g, ddf_ErrorType *Error)
+static ddf_MatrixPtr vectorList2Matrix(int n, const IntegerVectorList &g, ddf_ErrorType *Error)
 {
   ddf_MatrixPtr M=NULL;
   ddf_rowrange m_input,i;
@@ -24,7 +25,7 @@ static ddf_MatrixPtr vectorList2Matrix(const IntegerVectorList &g, ddf_ErrorType
   rep=ddf_Inequality; newformat=ddf_TRUE;
 
   m_input=g.size();
-  d_input=g.begin()->size()+1;
+  d_input=n+1;//g.begin()->size()+1;
 
   NT=ddf_Rational;
 
@@ -40,7 +41,7 @@ static ddf_MatrixPtr vectorList2Matrix(const IntegerVectorList &g, ddf_ErrorType
     }
     I++;
   }
-  
+
   successful=ddf_TRUE;
 
   return M;
@@ -75,7 +76,7 @@ bool LpSolverCdd::isFacet(const IntegerVectorList &g, IntegerVectorList::const_i
 
   //  ddf_init(val);
 
-  M=vectorList2Matrix(g, &err);
+  M=vectorList2Matrix(i->size(), g, &err);
 
   if (err!=ddf_NoError) goto _L99;
 
@@ -88,7 +89,7 @@ bool LpSolverCdd::isFacet(const IntegerVectorList &g, IntegerVectorList::const_i
   index=0;
   for(IntegerVectorList::const_iterator J=g.begin();J!=g.end()&&J!=i;J++){index++;}
   if(index==g.size())assert(0);
-  
+
 
   static ddf_Arow temp;
 
@@ -127,7 +128,7 @@ static void cddinitGmp()
 }
 
 
-static dd_MatrixPtr vectorList2MatrixGmp(const IntegerVectorList &g, dd_ErrorType *Error, int n=-1)
+static dd_MatrixPtr vectorList2MatrixGmp(int n, const IntegerVectorList &g, dd_ErrorType *Error)
 {
   dd_MatrixPtr M=NULL;
   dd_rowrange m_input,i;
@@ -151,6 +152,12 @@ static dd_MatrixPtr vectorList2MatrixGmp(const IntegerVectorList &g, dd_ErrorTyp
   d_input=n+1;
   if(m_input)
     {
+      if(g.begin()->size()!=n)
+	{
+	  AsciiPrinter(Stderr).printVectorList(g);
+	}
+
+
       assert(g.begin()->size()==n);
     }
 
@@ -168,7 +175,7 @@ static dd_MatrixPtr vectorList2MatrixGmp(const IntegerVectorList &g, dd_ErrorTyp
     }
     I++;
   }
-  
+
   successful=dd_TRUE;
 
   return M;
@@ -214,7 +221,7 @@ static dd_MatrixPtr vectorList2MatrixIncludingFirstColumnGmp(int n, const Intege
     for (j = 0; j < d_input; j++)dd_set_si(M->matrix[i][j],(*I)[j]);
     I++;
   }
-  
+
   successful=dd_TRUE;
 
   return M;
@@ -226,7 +233,7 @@ bool LpSolverCddGmp::hasHomogeneousSolution(int n, const IntegerVectorList &ineq
   int nrows=inequalities.size()+equations.size();
   if(nrows==0)return true;
 
-  dd_LPSolverType solver=dd_DualSimplex; 
+  dd_LPSolverType solver=dd_DualSimplex;
   dd_MatrixPtr A=NULL;
   dd_LPSolutionPtr lps1;
   dd_ErrorType err=dd_NoError;
@@ -251,7 +258,7 @@ bool LpSolverCddGmp::hasHomogeneousSolution(int n, const IntegerVectorList &ineq
   lp1=dd_MakeLPforInteriorFinding(lp);
   dd_LPSolve(lp1,solver,&err);
   if (err!=dd_NoError) goto _L99;
-  
+
   //  dd_WriteMatrix(Stderr,A);
   //  dd_WriteLP(Stderr,lp);
   //  dd_WriteLP(Stderr,lp1);
@@ -307,7 +314,7 @@ bool LpSolverCddGmp::isFacet(const IntegerVectorList &g, IntegerVectorList::cons
 
   //  dd_init(val);
 
-  M=vectorList2MatrixGmp(g, &err);
+  M=vectorList2MatrixGmp(i->size(),g, &err);
 
   if (err!=dd_NoError) goto _L99;
 
@@ -320,7 +327,7 @@ bool LpSolverCddGmp::isFacet(const IntegerVectorList &g, IntegerVectorList::cons
   index=0;
   for(IntegerVectorList::const_iterator J=g.begin();J!=g.end()&&J!=i;J++){index++;}
   if(index==g.size())assert(0);
-  
+
 
   static dd_Arow temp;
 
@@ -328,7 +335,7 @@ bool LpSolverCddGmp::isFacet(const IntegerVectorList &g, IntegerVectorList::cons
 
   ret= !dd_Redundant(M,index+1,temp,&err);
 
-  
+
   dd_FreeMatrix(M);
   dd_FreeArow(i->size()+1,temp);
 
@@ -340,13 +347,13 @@ bool LpSolverCddGmp::isFacet(const IntegerVectorList &g, IntegerVectorList::cons
 
 }
 
-int staticInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool strictlyPositive, IntegerVector const *equalitySet=0)
+int staticInteriorPoint(int n, mpq_t *point, const IntegerVectorList &g, bool strictlyPositive, IntegerVector const *equalitySet=0)
 {// copy-paste from testshoot.c in cdd
   //  AsciiPrinter(Stderr).printVectorList(g);
   //  fprintf(Stderr,"strictlyPositive:%i\n",strictlyPositive);
 
   //if(equalitySet)  AsciiPrinter(Stderr).printVector(*equalitySet);
-  dd_LPSolverType solver=dd_DualSimplex; 
+  dd_LPSolverType solver=dd_DualSimplex;
   dd_MatrixPtr A=NULL;
   dd_LPSolutionPtr lps1;
   dd_ErrorType err=dd_NoError;
@@ -362,10 +369,10 @@ int staticInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool strictlyP
       IntegerVectorList G=g;
       for(int i=0;i<n;i++)
 	G.push_back(IntegerVector::standardVector(n,i));
-      A=vectorList2MatrixGmp(G, &err);
+      A=vectorList2MatrixGmp(n,G, &err);
     }
   else
-    A=vectorList2MatrixGmp(g, &err);
+    A=vectorList2MatrixGmp(n, g, &err);
   if (err!=dd_NoError) goto _L99;
 
   if(equalitySet)
@@ -375,12 +382,12 @@ int staticInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool strictlyP
 	  dd_set_si(A->matrix[i][0],-1);
 
       assert(g.size()>=equalitySet->size());
-      
-      for(int i=0;i<equalitySet->size();i++)	
+
+      for(int i=0;i<equalitySet->size();i++)
 	if((*equalitySet)[i])set_addelem(A->linset, i+1);
     }
 
-  A->objective-dd_LPmax;
+  A->objective=dd_LPmax;
   lp=dd_Matrix2LP(A, &err);
   if (err!=dd_NoError) goto _L99;
 
@@ -392,7 +399,7 @@ int staticInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool strictlyP
   lp1=dd_MakeLPforInteriorFinding(lp);
   dd_LPSolve(lp1,solver,&err);
   if (err!=dd_NoError) goto _L99;
-  
+
   //  dd_WriteMatrix(Stderr,A);
   //  dd_WriteLP(Stderr,lp1);
 
@@ -419,13 +426,13 @@ int staticInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool strictlyP
   return 0;
 }
 
-int staticRelativeInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool strictlyPositive, IntegerVector const *equalitySet=0)
+int staticRelativeInteriorPoint(int n, mpq_t *point, const IntegerVectorList &g, bool strictlyPositive, IntegerVector const *equalitySet=0)
 {// copy-paste from testshoot.c in cdd
   //  AsciiPrinter(Stderr).printVectorList(g);
   //  fprintf(Stderr,"strictlyPositive:%i\n",strictlyPositive);
 
   //if(equalitySet)  AsciiPrinter(Stderr).printVector(*equalitySet);
-  dd_LPSolverType solver=dd_DualSimplex; 
+  dd_LPSolverType solver=dd_DualSimplex;
   dd_MatrixPtr A=NULL;
   dd_LPSolutionPtr lps1;
   dd_ErrorType err=dd_NoError;
@@ -434,17 +441,17 @@ int staticRelativeInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool s
 
   dd_LPPtr lp,lp1;
 
-  assert(g.begin()!=g.end());
+  //  assert(g.begin()!=g.end());
   if(strictlyPositive)
     {
-      int n=g.begin()->size();
+      //  int n=g.begin()->size();
       IntegerVectorList G=g;
       for(int i=0;i<n;i++)
 	G.push_back(IntegerVector::standardVector(n,i));
-      A=vectorList2MatrixGmp(G, &err);
+      A=vectorList2MatrixGmp(n, G, &err);
     }
   else
-    A=vectorList2MatrixGmp(g, &err);
+    A=vectorList2MatrixGmp(n, g, &err);
   if (err!=dd_NoError) goto _L99;
 
   if(equalitySet)
@@ -454,12 +461,12 @@ int staticRelativeInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool s
 	  dd_set_si(A->matrix[i][0],-1);
 
       assert(g.size()>=equalitySet->size());
-      
-      for(int i=0;i<equalitySet->size();i++)	
+
+      for(int i=0;i<equalitySet->size();i++)
 	if((*equalitySet)[i])set_addelem(A->linset, i+1);
     }
 
-  A->objective-dd_LPmax;
+  A->objective=dd_LPmax;
   lp=dd_Matrix2LP(A, &err);
   if (err!=dd_NoError) goto _L99;
 
@@ -471,7 +478,7 @@ int staticRelativeInteriorPoint(mpq_t *point, const IntegerVectorList &g, bool s
   lp1=dd_MakeLPforInteriorFinding(lp);
   dd_LPSolve(lp1,solver,&err);
   if (err!=dd_NoError) goto _L99;
-  
+
   //  dd_WriteMatrix(Stderr,A);
   //  dd_WriteLP(Stderr,lp1);
 
@@ -505,8 +512,8 @@ bool LpSolverCddGmp::hasInteriorPoint(const IntegerVectorList &g, bool strictlyP
   mpq_t *point = new mpq_t [n];
   for(int i=0;i<n;i++)mpq_init(point[i]);
 
-  int ret=staticInteriorPoint(point,g,strictlyPositive,equalitySet);
-  
+  int ret=staticInteriorPoint(n, point,g,strictlyPositive,equalitySet);
+
   // fprintf(Stderr,"%i\n",ret);
 
   for(int i=0;i<n;i++)mpq_clear(point[i]);
@@ -525,14 +532,14 @@ IntegerVector arrayToIntegerVector(mpq_t *point, int n)
     {
       int den;
       int num;
-      
+
       if((!mpz_fits_sint_p(mpq_denref(point[j])))||(!mpz_fits_sint_p(mpq_numref(point[j]))))
 	{
 	  fprintf(stderr,"INTEGER OVERFLOW IN POLYHEDRAL COMPUTATION\n");
-	  assert(0); 
+	  assert(0);
 	}
       den=mpz_get_si(mpq_denref(point[j]));
-      num=mpz_get_si(mpq_numref(point[j]));    
+      num=mpz_get_si(mpq_numref(point[j]));
 
       assert(den==1);
       ret[j]=num;
@@ -547,23 +554,26 @@ void scaleToIntegerVector(mpq_t *point, int n)
   mpz_t gcd;
   mpz_init_set_ui(lcm, 1);
   mpz_init_set_ui(gcd, 0);
-  
+
   for(int j=0;j<n;j++)
     {
       mpz_lcm(lcm,lcm,mpq_denref(point[j]));
       mpz_gcd(gcd,gcd,mpq_numref(point[j]));
     }
-  
+
   if(mpz_sgn(gcd)!=0)
+if(mpz_cmp(gcd,lcm)!=0)
     {
+      assert(mpz_sgn(gcd)>0);
       mpq_t scale;
       mpq_init(scale);
       mpq_set_den(scale,gcd);
-      mpq_set_num(scale,lcm);    
+      mpq_set_num(scale,lcm);
+      mpq_canonicalize(scale); //according to the gmp manual this call is needed, although it slows things down a bit
       for(int j=0;j<n;j++)
 	{
 	  mpq_mul(point[j],point[j],scale);
-	}    
+	}
       mpq_clear(scale);
     }
   mpz_clear(lcm);
@@ -571,14 +581,14 @@ void scaleToIntegerVector(mpq_t *point, int n)
 }
 
 
-IntegerVector LpSolverCddGmp::relativeInteriorPoint(const IntegerVectorList &g, IntegerVector const *equalitySet)
+IntegerVector LpSolverCddGmp::relativeInteriorPoint(int n, const IntegerVectorList &g, IntegerVector const *equalitySet)
 {
-  assert(!g.empty());
-  int n=g.begin()->size();
+  //  assert(!g.empty());
+  //  int n=g.begin()->size();
   mpq_t *point = new mpq_t [n];
   for(int i=0;i<n;i++)mpq_init(point[i]);
 
-  int ret=staticRelativeInteriorPoint(point,g,false,equalitySet);
+  int ret=staticRelativeInteriorPoint(n, point,g,false,equalitySet);
 
   assert(ret>=0);//-- any cone has a relative interior point
   //  if (ret>0){
@@ -588,7 +598,7 @@ IntegerVector LpSolverCddGmp::relativeInteriorPoint(const IntegerVectorList &g,
     scaleToIntegerVector(point,n);
     //   fprintf(stderr,"TEST2\n");
   }
-  
+
 
   IntegerVector result=arrayToIntegerVector(point,n);
 
@@ -606,14 +616,14 @@ bool LpSolverCddGmp::interiorPoint(const IntegerVectorList &g, IntegerVector &re
   mpq_t *point = new mpq_t [n];
   for(int i=0;i<n;i++)mpq_init(point[i]);
 
-  int ret=staticInteriorPoint(point,g,strictlyPositive,equalitySet);
+  int ret=staticInteriorPoint(n, point,g,strictlyPositive,equalitySet);
 
   //  if (ret>0){
   if (ret>=0)
   {
     scaleToIntegerVector(point,n);
   }
-  
+
 
   result=arrayToIntegerVector(point,n);
   //  if (ret>0){
@@ -625,9 +635,9 @@ bool LpSolverCddGmp::interiorPoint(const IntegerVectorList &g, IntegerVector &re
     }
     fprintf(f,")\n");
   }
-  if (ret<0) 
+  if (ret<0)
     fprintf(f,"The feasible region is empty.\n");
-  if (ret==0) 
+  if (ret==0)
     fprintf(f,"The feasible region is nonempty but has no interior point.\n");
   */
 
@@ -684,7 +694,7 @@ IntegerVectorList::const_iterator LpSolverCddGmp::shoot(const IntegerVectorList
   //  AsciiPrinter(Stderr).printVectorList(g);
 
 
-  int ret=staticInteriorPoint(point,g,true);
+  int ret=staticInteriorPoint(n, point,g,true);
 
   //fprintf(Stderr,"Interior point\n");
   //  AsciiPrinter(Stderr).printIntegerVector(point);
@@ -746,6 +756,7 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
   //  AsciiPrinter(Stderr).printVectorList(g);
   //  fprintf(Stderr,"strictlyPositive:%i\n",strictlyPositive);
 
+
   IntegerVectorList g=g_;
 
 
@@ -759,7 +770,7 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
   for(int i=0;i<dimension;i++)
     g.push_back(IntegerVector::standardVector(dimension,i));
 
-  dd_LPSolverType solver=dd_DualSimplex; 
+  dd_LPSolverType solver=dd_DualSimplex;
   dd_MatrixPtr A=NULL;
   dd_LPSolutionPtr lps1;
   dd_ErrorType err=dd_NoError;
@@ -768,18 +779,18 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
 
   dd_LPPtr lp;
 
-  A=vectorList2MatrixGmp(g, &err);
+  A=vectorList2MatrixGmp(dimension, g, &err);
 
   for(int i=0;i<dim2;i++)set_addelem(A->linset, i+1);//???
   // set objective function
   for (int i = 0; i < dimension; i++)
     dd_set_si(A->rowvec[i+1],-1);
-  
+
 
   // set right hand side
   for (int i = 0; i < dimension; i++)
     dd_set_si(A->matrix[i+dim2][0],-1);
-  
+
 
   if (err!=dd_NoError) goto _L99;
   A->objective=dd_LPmax;
@@ -789,7 +800,7 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
     {
       for(int j=0;j<dimension;j++)
 	dd_WriteNumber(f,point[j]);
-	
+
       fprintf(Stderr,"\n");
     }
   */
@@ -800,27 +811,37 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
   lp=dd_Matrix2LP(A, &err);
   if (err!=dd_NoError) goto _L99;
 
+//  dd_WriteLP(Stderr,lp);
   /* Find an interior point with cdd LP library. */
 
   dd_LPSolve(lp,solver,&err);
-  if (err!=dd_NoError) goto _L99;
-  
 
-  dd_WriteLPResult(stdout,lp,err);
+
+  if (err!=dd_NoError) goto _L99;
+
+  //  IF INFEASIBLE RETURN FALSE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+  if(lp->LPS==dd_Inconsistent)
+    {
+      dd_FreeLPData(lp);
+      dd_FreeMatrix(A);
+
+      return false;
+    }
+//  dd_WriteLPResult(stdout,lp,err);
 
   /* Write an interior point. */
   lps1=dd_CopyLPSolution(lp);
 
   if (dd_Positive(lps1->optvalue))
     {
-      fprintf(Stderr,"Returning false\n");
+//      fprintf(Stderr,"Returning false\n");
       return false;
     }
 
 
 
 
-  {
+/*  {
     fprintf(Stderr,"(");
     for (int j=1; j <(lps1->d); j++) {
       if(j!=1)fprintf(Stderr,", ");
@@ -828,6 +849,7 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
     }
     fprintf(Stderr,")\n");
   }
+*/
 
   //transform into integer vectors
   {
@@ -838,7 +860,7 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
     mpz_t gcd;
     mpz_init_set_ui(lcm, 1);
     mpz_init_set_ui(gcd, 0);
-    
+
     for(int j=0;j<n;j++)
       {
 	mpz_lcm(lcm,lcm,mpq_denref(point[j]));
@@ -848,11 +870,11 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
     mpq_t scale;
     mpq_init(scale);
     mpq_set_den(scale,gcd);
-    mpq_set_num(scale,lcm);    
+    mpq_set_num(scale,lcm);
     for(int j=0;j<n;j++)
       {
 	mpq_mul(point[j],point[j],scale);
-      }    
+      }
 
     mpz_clear(lcm);
     mpz_clear(gcd);
@@ -860,7 +882,7 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
 
 
 
-  {
+/*  {
     fprintf(Stderr,"(");
     for (int j=1; j <(lps1->d); j++) {
       if(j!=1)fprintf(Stderr,", ");
@@ -868,14 +890,14 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
     }
     fprintf(Stderr,")\n");
   }
-
+*/
   for (int j=1; j <(lps1->d); j++)
     {
       int den;
       int num;
-      
+
       den=mpz_get_si(mpq_denref(lps1->sol[j]));
-      num=mpz_get_si(mpq_numref(lps1->sol[j]));    
+      num=mpz_get_si(mpq_numref(lps1->sol[j]));
 
       assert(den==1);
       if(result)
@@ -903,6 +925,9 @@ bool LpSolverCddGmp::positiveVectorInKernel(const IntegerVectorList &g_, Integer
 #include "subspace.h"
 int LpSolverCddGmp::rankOfMatrix(const IntegerVectorList &g_)
 {
+  if(g_.size()==0)return 0;
+  FieldMatrix temp=integerMatrixToFieldMatrix(rowsToIntegerMatrix(g_),Q);
+  return temp.reduceAndComputeRank();
   return Subspace(g_).dimension();
   dd_rowset r=NULL;
   int result;
@@ -916,14 +941,14 @@ int LpSolverCddGmp::rankOfMatrix(const IntegerVectorList &g_)
   /*  for(int i=0;i<dimension;i++)
     g.push_back(IntegerVector::standardVector(dimension,i));
   */
-  dd_LPSolverType solver=dd_DualSimplex; 
+  dd_LPSolverType solver=dd_DualSimplex;
   dd_MatrixPtr A=NULL;
   dd_ErrorType err=dd_NoError;
   //  int ret=0;
   cddinitGmp();
 
 
-  A=vectorList2MatrixGmp(g, &err);
+  A=vectorList2MatrixGmp(dimension, g, &err);
 
   //  dd_WriteMatrix(Stderr,A);
 
@@ -933,7 +958,7 @@ int LpSolverCddGmp::rankOfMatrix(const IntegerVectorList &g_)
   // set objective function
   for (int i = 0; i < dimension; i++)
     dd_set_si(A->rowvec[i+1],-1);
-  
+
 
   // set right hand side
   /*for (int i = 0; i < dimension; i++)
@@ -943,7 +968,7 @@ int LpSolverCddGmp::rankOfMatrix(const IntegerVectorList &g_)
   if (err!=dd_NoError) goto _L99;
   A->objective=dd_LPmax;
 
-  fprintf(Stderr,"rank of matrix matrix:\n");
+  //fprintf(Stderr,"rank of matrix matrix:\n");
 
   //  dd_WriteMatrix(Stderr,A);
 
@@ -952,7 +977,7 @@ int LpSolverCddGmp::rankOfMatrix(const IntegerVectorList &g_)
 
   result=dim2-set_card(r);
 
-  fprintf(Stderr,"dim2==%i set_card(r)==%i\n",dim2,set_card(r));
+//  fprintf(Stderr,"dim2==%i set_card(r)==%i\n",dim2,set_card(r));
 
   set_free(r);
 
@@ -971,7 +996,7 @@ int LpSolverCddGmp::rankOfMatrix(const IntegerVectorList &g_)
 static void dd_ComputeAinc(dd_PolyhedraPtr poly)
 {
 /* This generates the input incidence array poly->Ainc, and
-   two sets: poly->Ared, poly->Adom. 
+   two sets: poly->Ared, poly->Adom.
 */
   dd_bigrange k;
   dd_rowrange i,m1;
@@ -985,7 +1010,13 @@ static void dd_ComputeAinc(dd_PolyhedraPtr poly)
 
   M=dd_CopyOutput(poly);
   poly->n=M->rowsize;
-  m1=poly->m1;  
+  m1=poly->m1;
+
+
+  /*  fprintf(Stderr,"m1:%i\n",m1);
+  fprintf(Stderr,"n:%i\n",poly->n);
+  fprintf(Stderr,"m:%i\n",poly->m);
+  */
    /* this number is same as poly->m, except when
       poly is given by nonhomogeneous inequalty:
       !(poly->homogeneous) && poly->representation==Inequality,
@@ -993,8 +1024,8 @@ static void dd_ComputeAinc(dd_PolyhedraPtr poly)
    */
   poly->Ainc=(set_type*)calloc(m1, sizeof(set_type));
   for(i=1; i<=m1; i++) set_initialize(&(poly->Ainc[i-1]),poly->n);
-  set_initialize(&(poly->Ared), m1); 
-  set_initialize(&(poly->Adom), m1); 
+  set_initialize(&(poly->Ared), m1);
+  set_initialize(&(poly->Adom), m1);
 
   for (k=1; k<=poly->n; k++){
     for (i=1; i<=poly->m; i++){
@@ -1017,7 +1048,7 @@ static void dd_ComputeAinc(dd_PolyhedraPtr poly)
   for (i=1; i<=m1; i++){
     if (set_card(poly->Ainc[i-1])==M->rowsize){
       set_addelem(poly->Adom, i);
-    }  
+    }
   }
   for (i=m1; i>=1; i--){
     if (set_card(poly->Ainc[i-1])==0){
@@ -1026,7 +1057,7 @@ static void dd_ComputeAinc(dd_PolyhedraPtr poly)
     }else {
       redundant=dd_FALSE;
       for (k=1; k<=m1; k++) {
-        if (k!=i && !set_member(k, poly->Ared)  && !set_member(k, poly->Adom) && 
+        if (k!=i && !set_member(k, poly->Ared)  && !set_member(k, poly->Adom) &&
             set_subset(poly->Ainc[i-1], poly->Ainc[k-1])){
           if (!redundant){
             redundant=dd_TRUE;
@@ -1050,7 +1081,9 @@ IntegerVectorList LpSolverCddGmp::extremeRaysInequalityIndices(const IntegerVect
 
   int dim2=g.size();
 
-  assert(g.size()!=0);
+  //  assert(g.size()!=0);
+  if(g.size()==0)return IntegerVectorList();
+
   int dimension=g.begin()->size();
 
   dd_MatrixPtr A=NULL;
@@ -1058,37 +1091,60 @@ IntegerVectorList LpSolverCddGmp::extremeRaysInequalityIndices(const IntegerVect
 
   cddinitGmp();
 
-  A=vectorList2MatrixGmp(g, &err);
+  A=vectorList2MatrixGmp(dimension, g, &err);
 
   //  dd_WriteMatrix(Stderr,A);
 
   dd_PolyhedraPtr poly;
   poly=dd_DDMatrix2Poly2(A, dd_LexMin, &err);
 
+
+  //  dd_WritePolyFile(Stderr,poly);
+
   //  dd_WriteIncidence(Stderr,poly);
   if (poly->child==NULL || poly->child->CompStatus!=dd_AllFound) assert(0);
   if (poly->AincGenerated==dd_FALSE) dd_ComputeAinc(poly);
-  //  dd_WriteIncidence(Stderr,poly);
+
+
+  //    dd_WriteIncidence(Stderr,poly);
 
   IntegerVectorList ret;
 
+  //  fprintf(Stderr,"%i %i\n",poly->n,poly->m1);
+
+  /*
+    How do we interpret the cddlib output?  For a long ting gfan has
+    been using poly->n as the number of rays of the cone and thus
+    returned sets of indices that actually gave the lineality
+    space. The mistake was then caught later in PolyhedralCone. On Feb
+    17 2009 gfan was changed to check the length of each set to make
+    sure that it does not specify the lineality space and only return
+    those sets giving rise to rays.  This does not seem to be the best
+    strategy and might even be wrong.
+   */
+
+
   for (int k=1; k<=poly->n; k++)
+    //for (int k=1; k<=poly->m1; k++)
     {
       int length=0;
       for (int i=1; i<=poly->m1; i++)
 	if(set_member(k,poly->Ainc[i-1]))length++;
-      IntegerVector v(length);
-      int j=0;
-      for (int i=1; i<=poly->m1; i++)
-	if(set_member(k,poly->Ainc[i-1]))v[j++]=i-1;
-      ret.push_back(v);
+      if(length!=dim2)
+	{
+	  IntegerVector v(length);
+	  int j=0;
+	  for (int i=1; i<=poly->m1; i++)
+	    if(set_member(k,poly->Ainc[i-1]))v[j++]=i-1;
+	  ret.push_back(v);
+	}
     }
 
   //  AsciiPrinter(Stderr).printVectorList(ret);
   //  dd_WriteIncidence(Stderr,poly);
 
   dd_FreeMatrix(A);
-  dd_FreePolyhedra(poly);  
+  dd_FreePolyhedra(poly);
 
   return ret;
  _L99:
@@ -1121,13 +1177,13 @@ void LpSolverCddGmp::removeRedundantRows(IntegerVectorList *inequalities, Intege
   if(numberOfRows==0)return;//the full space, so description is alredy irredundant
   assert(numberOfRows>0);
 
-  dd_LPSolverType solver=dd_DualSimplex; 
+  dd_LPSolverType solver=dd_DualSimplex;
   dd_MatrixPtr A=NULL;
   dd_ErrorType err=dd_NoError;
 
   cddinitGmp();
 
-  A=vectorList2MatrixGmp(g, &err);
+  A=vectorList2MatrixGmp(g.begin()->size(),g, &err);
 
   for(int i=numberOfInequalities;i<numberOfRows;i++)
     set_addelem(A->linset,i+1);
@@ -1138,7 +1194,7 @@ void LpSolverCddGmp::removeRedundantRows(IntegerVectorList *inequalities, Intege
   // set objective function
   //  for (int i = 0; i < dimension; i++)
   //    dd_set_si(A->rowvec[i+1],-1);
-  
+
 
   // set right hand side
   /*for (int i = 0; i < dimension; i++)
@@ -1182,12 +1238,12 @@ void LpSolverCddGmp::removeRedundantRows(IntegerVectorList *inequalities, Intege
 	{
 	  mpq_t *point = new mpq_t [n];
 	  for(int j=0;j<n;j++)mpq_init(point[j]);
-	  
+
 	  for(int j=0;j<n;j++)mpq_set(point[j],A->matrix[i][j+1]);
 
 	  IntegerVector v=arrayToIntegerVector(point, n);
 	  //  AsciiPrinter(Stderr).printVector(v);
-	  
+
 	  for(int j=0;j<n;j++)mpq_clear(point[j]);
 	  delete [] point;
 	  if(set_member(i+1,A->linset))
@@ -1210,7 +1266,7 @@ void LpSolverCddGmp::removeRedundantRows(IntegerVectorList *inequalities, Intege
 	    }
 	  i++;
 	}
-      
+
       for(IntegerVectorList::iterator i=equalities->begin();i!=equalities->end();index++)
 	{
 	  int i2=newpos[index+1];
@@ -1230,12 +1286,12 @@ void LpSolverCddGmp::removeRedundantRows(IntegerVectorList *inequalities, Intege
 
   if(A->rowsize!=newLin.size()+newIn.size())
     {
-      fprintf(stderr,"A->rowsize: %i\n",A->rowsize);
+      fprintf(stderr,"A->rowsize: %i\n",(int)A->rowsize);
       fprintf(stderr,"newLin.size(): %i\n",newLin.size());
       fprintf(stderr,"newIn.size(): %i\n",newIn.size());
 
       dd_WriteMatrix(Stderr,A);
-  
+
       AsciiPrinter(Stderr).printVectorList(newLin);
       AsciiPrinter(Stderr).printVectorList(newIn);
 
@@ -1272,7 +1328,7 @@ static dd_MatrixPtr vectorLists2MatrixGmp(int n, IntegerVectorList const &inequa
 
   cddinitGmp();
 
-  A=vectorList2MatrixGmp(g, err, n);
+  A=vectorList2MatrixGmp(n, g, err);
 
   for(int i=numberOfInequalities;i<numberOfRows;i++)
     set_addelem(A->linset,i+1);
@@ -1297,11 +1353,11 @@ static IntegerVectorList getConstraints(dd_MatrixPtr A, bool returnEquations)
       if(isEquation==returnEquations)
 	{
 	  for(int j=0;j<n;j++)mpq_set(point[j],A->matrix[i][j+1]);
-      
+
 	  scaleToIntegerVector(point,n);
 	  IntegerVector v=arrayToIntegerVector(point, n);
       //  AsciiPrinter(Stderr).printVector(v);
-	  
+
 	  ret.push_back(v);
 	}
     }
@@ -1348,18 +1404,18 @@ void LpSolverCddGmp::dual(int n, const IntegerVectorList &inequalities, const In
   *dualEquations=getConstraints(A2,true);
 
   dd_FreeMatrix(A2);
-  
+
   //  AsciiPrinter(Stderr).printVectorList(*dualInequalities);
   //  AsciiPrinter(Stderr).printVectorList(*dualEquations);
 
-  
+
   //  assert(0);
 
   //  AsciiPrinter(Stderr).printVectorList(ret);
   //  dd_WriteIncidence(Stderr,poly);
 
   dd_FreeMatrix(A);
-  dd_FreePolyhedra(poly);  
+  dd_FreePolyhedra(poly);
 
   return;
  _L99:
