@@ -45,3 +45,23 @@ FloatMatrix integerToFloatMatrix(IntegerMatrix const &m)
 
   return ret;
 }
+
+
+IntegerVector flattenMatrix(IntegerMatrix const &m)
+{
+  IntegerVector ret(m.getHeight()*m.getWidth());
+
+  for(int i=0;i<m.getHeight();i++)
+    for(int j=0;j<m.getWidth();j++)
+      ret[i*m.getWidth()+j]=m[i][j];
+
+  return ret;
+}
+
+
+#include "linalg.h"
+
+int rank(IntegerMatrix const &m)
+{
+  return integerMatrixToFieldMatrix(m,Q).rank();
+}
