@@ -1,9 +1,15 @@
 #include "polynomial.h"
 
+#include <sstream>
+#include <iostream>
+
+#include "division.h"
 #include "printer.h"
 #include "timer.h"
+#include "linalg.h"
+#include "wallideal.h"
 
-static Timer polynomialTimer("Polynomial subtraction",10);
+//static Timer polynomialTimer("Polynomial subtraction",10);
 //static Timer polynomialTimer1("Polynomial monomial multiplication",1);
 
 //-----------------------------------------
@@ -16,6 +22,7 @@ Polynomial::Polynomial(const Term &t):
   theRing(t.getRing())
 {
   terms[t.m]=t.c;
+  isMarkedBool=true;
 }
 
 
@@ -24,8 +31,24 @@ Polynomial::Polynomial(PolynomialRing const &r):
   sugar(0),
   marked(r)
 {
+	isMarkedBool=false;
 }
 
+Polynomial Polynomial::half(bool secondHalf)const
+{
+  Polynomial ret(theRing);
+  int numberOfTerms=terms.size();
+  int splitIndex=numberOfTerms/2;
+  int I=0;
+  for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++,I++)
+    if((secondHalf)!=(I<splitIndex))
+    {
+      ret.terms.insert(ret.terms.end(),*i);
+    }
+  return ret;
+}
+
+
 void Polynomial::mark(TermOrder const &termOrder)
 {
   TermMap::iterator i=terms.begin();
@@ -38,6 +61,7 @@ void Polynomial::mark(TermOrder const &termOrder)
 	if(termOrder(best.m.exponent,i->first.exponent))best=Term(i->second,i->first);
       marked=best;
     }
+  isMarkedBool=true;
 }
 
 
@@ -48,6 +72,7 @@ void Polynomial::mark(Monomial const &monomial)
     if(i->first.exponent==monomial.exponent)
       {
 	marked=Term(i->second,monomial);
+	isMarkedBool=true;
 	return;
       }
   fprintf(Stderr,"Monomial ");
@@ -59,9 +84,16 @@ void Polynomial::mark(Monomial const &monomial)
 }
 
 
+void Polynomial::copyMarking(Polynomial const &p)
+{
+  mark(p.marked.m);
+}
+
+
 bool Polynomial::checkMarking(TermOrder const &termOrder)const
 {
-  for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+	if(!isMarked())return false;
+	for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
     {
       //      if(!termOrder(i->first.exponent,marked.m.exponent))return false;
       if(termOrder(marked.m.exponent,i->first.exponent))return false;
@@ -70,8 +102,24 @@ bool Polynomial::checkMarking(TermOrder const &termOrder)const
 }
 
 
+bool Polynomial::isHomogeneous(IntegerVector const &v)const
+{
+  if(isZero())return true;
+  return degree(v)==-degree(-v);
+}
+
+
+bool Polynomial::isHomogeneous()const
+{
+  if(isZero())return true;
+  IntegerVector v=IntegerVector::allOnes(theRing.getNumberOfVariables());
+  return degree(v)==-degree(-v);
+}
+
+
 void Polynomial::scaleMarkedCoefficientToOne()
 {
+	assert(isMarked());
   FieldElement a=marked.c.inverse();
   Monomial k(theRing);
   k.exponent=IntegerVector(getNumberOfVariables());
@@ -81,7 +129,7 @@ void Polynomial::scaleMarkedCoefficientToOne()
 }
 
 
-int Polynomial::getNumberOfVariables()const 
+int Polynomial::getNumberOfVariables()const
 {
   TermMap::const_iterator i=terms.begin();
   if(i==terms.end())return 0;
@@ -102,7 +150,7 @@ void Polynomial::changeNumberOfVariables(int n)
       FieldElement e=i->second;
       q+=Term(e,Monomial(theRing,v));
     }
-  marked.m.exponent.resize(n);
+  if(isMarked())marked.m.exponent.resize(n);
   terms=q.terms;
 }
 
@@ -151,7 +199,7 @@ void Polynomial::operator+=(const Polynomial &p)
   if(p.getSugar()>sugar)sugar=p.getSugar();
   // fast addition
   //  TimerScope ts(&polynomialTimer);
-  
+
   TermMap::iterator i=terms.lower_bound(p.terms.begin()->first);
 
   for(TermMap::const_iterator j=p.terms.begin();j!=p.terms.end();j++)
@@ -202,8 +250,8 @@ void Polynomial::operator-=(const Polynomial &p)
   if(p.terms.empty())return; //added May 7 2005
   if(p.getSugar()>sugar)sugar=p.getSugar();
   // fast subtraction
-  TimerScope ts(&polynomialTimer);
-  
+  //TimerScope ts(&polynomialTimer);
+
   TermMap::iterator i=terms.lower_bound(p.terms.begin()->first);
 
   for(TermMap::const_iterator j=p.terms.begin();j!=p.terms.end();j++)
@@ -220,6 +268,7 @@ void Polynomial::operator-=(const Polynomial &p)
 	      FieldElement c=i->second-j->second;
 	      if(c.isZero())
 		{
+		  //		  if(i->fist.exponent==marked.exponent)marked=Term);
 		  TermMap::iterator oldI=i;
 		  i++;
 		  terms.erase(oldI);
@@ -246,7 +295,7 @@ void Polynomial::operator-=(const Polynomial &p)
         }
       else
         terms[i->first]=-i->second;
-    } 
+    }
   */
 }
 
@@ -271,7 +320,7 @@ void Polynomial::operator*=(const Term &t)
 {
   sugar+=t.m.exponent.sum();
   // faster multiplication
-  Polynomial p(theRing);  
+  Polynomial p(theRing);
   for(TermMap::iterator i=terms.begin();i!=terms.end();i++)
     {
       FieldElement prod=i->second;
@@ -279,7 +328,7 @@ void Polynomial::operator*=(const Term &t)
       p.terms.insert(p.terms.end(),TermMap::value_type(Monomial(theRing,i->first.exponent+t.m.exponent),prod));
     }
   terms=p.terms;
-  
+
   // slow multiplication
   /*  Polynomial p;
   for(TermMap::iterator i=terms.begin();i!=terms.end();i++)
@@ -296,7 +345,7 @@ void Polynomial::operator*=(const Term &t)
 void Polynomial::operator*=(const Monomial &m)
 {
   sugar+=m.exponent.sum();
-  Polynomial p(theRing);  
+  Polynomial p(theRing);
   for(TermMap::iterator i=terms.begin();i!=terms.end();i++)
     {
       FieldElement prod=i->second;
@@ -318,12 +367,36 @@ void Polynomial::operator*=(Polynomial const &p)
 {
   Polynomial r(theRing);
 
+#if 0
+  if(terms.size()>p.terms.size())
+    {Polynomial q=p;q*=*this;*this=q;}
+  else if(isZero())
+    *this=Polynomial(theRing);
+  else if(terms.size()==1)
+    *this=p*Term(terms.begin()->second,terms.begin()->first);
+  else
+    {
+      Polynomial A=half(true);
+      Polynomial B=half(false);
+      A*=p;
+      B*=p;
+      *this=A+B;
+    }
+#else
+  if(terms.size()>p.terms.size())
+    {
+      Polynomial q=p;q*=*this;*this=q;
+    }
+  else
+  {
   for(TermMap::iterator i=terms.begin();i!=terms.end();i++)
     {
       r+=p*Term(i->second,i->first);
     }
   *this=r;
-}
+  }
+#endif
+  }
 
 Polynomial operator*(const Polynomial &p, Term const &t)
 {
@@ -381,6 +454,15 @@ IntegerVector Polynomial::exponentsSum()const
   return sum;
 }
 
+IntegerVector Polynomial::greatestCommonMonomialDivisor()const
+{
+  assert(!isZero());
+  IntegerVector ret=terms.begin()->first.exponent;
+  for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+    ret=min(ret,i->first.exponent);
+  return ret;
+}
+
 bool Polynomial::isMonomial()const
 {
   return terms.size()==1;//could it be faster to compare begin and end iterators?
@@ -406,7 +488,6 @@ int64 Polynomial::degree(IntegerVector const &w)const
       if(first||dotLong(i->first.exponent,w)>d)d=dotLong(i->first.exponent,w);
       first=false;
     }
-
   assert(!first);
 
   return d;
@@ -424,7 +505,7 @@ Polynomial Polynomial::homogenization(PolynomialRing const &newRing, IntegerVect
     degree=totalDegree();
 
   IntegerVector m;
-  
+
   for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
     {
       IntegerVector v=i->first.exponent;
@@ -441,20 +522,54 @@ Polynomial Polynomial::homogenization(PolynomialRing const &newRing, IntegerVect
 
       v+=a;
       ret+=Term(i->second,Monomial(newRing,v));
-      if(marked.m.exponent==i->first.exponent)m=v;
+      if(isMarked())if(marked.m.exponent==i->first.exponent)m=v;
     }
 
   if(isMarked())
     {
-      assert(m.size()==newRing.getNumberOfVariables());
-      ret.mark(Monomial(newRing,m));
-    }
+	  IntegerVector v=m;
+/*	  if(m.size()+1==newRing.getNumberOfVariables())
+	  {
+	  v.grow(m.size()+1);
+	  v[m.size()]=degree-v.sum();
+	  ret.mark(Monomial(newRing,v));
+	  }*/
+	  if(m.size()==newRing.getNumberOfVariables())
+	  {
+	  ret.mark(Monomial(newRing,v));
+	  }
+	  //      assert(m.size()==newRing.getNumberOfVariables());
+//      ret.mark(Monomial(newRing,m));
+	  assert(ret.isMarked());
 
+    }
   return ret;
 }
 
 
-Polynomial Polynomial::derivative()const
+Polynomial Polynomial::torusAct(FieldVector const &w)const
+{
+  Polynomial ret=*this;
+  int n=theRing.getNumberOfVariables();
+  assert(w.size()==n);
+  for(TermMap::iterator i=ret.terms.begin();i!=ret.terms.end();i++)
+    {
+      FieldElement c=theRing.getField().zHomomorphism(1);
+      for(int j=0;j<n;j++)
+	for(int k=0;k<i->first.exponent[j];k++)
+	  c*=w[j];
+      i->second=i->second*c;
+    }
+
+  if(isMarked())
+    {
+      ret.mark(marked.m);
+    }
+  return ret;
+}
+
+
+/*Polynomial Polynomial::derivative()const
 {
   Polynomial ret(theRing);
 
@@ -469,6 +584,24 @@ Polynomial Polynomial::derivative()const
 	}
     }
 
+  return ret;
+}*/
+
+
+Polynomial Polynomial::derivative(int j)const
+{
+  Polynomial ret(theRing);
+
+  for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+    {
+      IntegerVector v=i->first.exponent;
+      if(v[j]!=0)
+      {
+    	  v[j]--;
+    	  ret+=Term(i->second*theRing.getField().zHomomorphism(v[j]+1),Monomial(theRing,v));
+      }
+    }
+
   return ret;
 }
 
@@ -483,10 +616,32 @@ Polynomial Polynomial::deHomogenization()const
 }
 
 
+Polynomial Polynomial::deHomogenizationInSameRing()const
+{
+  Polynomial ret(getRing());
+  int n=getRing().getNumberOfVariables();
+  assert(n>0);
+  for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+    {
+      IntegerVector v=i->first.exponent;
+      v[n-1]=0;
+      FieldElement e=i->second;
+      ret+=Term(e,Monomial(theRing,v));
+    }
+  if(isMarked())
+  {
+	  ret.marked.m.exponent=marked.m.exponent.subvector(0,n-1);
+  ret.isMarkedBool=true;
+  }
+	  return ret;
+}
+
+
 int Polynomial::numberOfVariablesInRing()const
 {
-  assert(terms.size()!=0);
-  return terms.begin()->first.exponent.size();
+  return theRing.getNumberOfVariables();
+//  assert(terms.size()!=0);
+//  return terms.begin()->first.exponent.size();
 }
 
 
@@ -495,19 +650,24 @@ void Polynomial::saturate(int variableNum)//does not compute sugar
   if(!terms.empty())
     {
       IntegerVector smallest=terms.begin()->first.exponent;
-      
+
       for(TermMap::iterator i=terms.begin();i!=terms.end();i++)
 	smallest=min(smallest,i->first.exponent);
-      
+
       if(variableNum!=-1)
 	{
 	  for(int j=0;j<smallest.size();j++)if(j!=variableNum)smallest[j]=0;
 	}
 
-      Polynomial p(theRing);  
+      Polynomial p(theRing);
       for(TermMap::iterator i=terms.begin();i!=terms.end();i++)
 	p.terms.insert(p.terms.end(),TermMap::value_type(Monomial(theRing,i->first.exponent-smallest),i->second));
       terms=p.terms;
+
+      if(isMarked())
+	{
+	  marked.m.exponent-=smallest;
+	}
     }
 }
 
@@ -525,7 +685,8 @@ int Polynomial::getSugar()const
 
 bool Polynomial::isMarked()const
 {
-  return marked.m.exponent.size()!=0;
+	return isMarkedBool;
+//  return marked.m.exponent.size()!=0;
 }
 
 
@@ -542,9 +703,9 @@ bool Polynomial::isValid(int numberOfVariables)const
 	      return false;
 	    }
 	}
-      if(marked.m.exponent.size())
+      if(isMarked())
 	{
-	  assert(marked.m.exponent.size()==numberOfVariables); 
+	  assert(marked.m.exponent.size()==numberOfVariables);
 	}
     }
   return true;
@@ -582,21 +743,45 @@ int Polynomial::maximalIndexOfVariableInSupport()const
 }
 
 
-Polynomial Polynomial::embeddedInto(PolynomialRing const &r2)const
+IntegerVector Polynomial::usedVariables()const
+{
+	int n=theRing.getNumberOfVariables();
+	IntegerVector ret(n);
+	for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+	{
+		ret=max(ret,i->first.exponent.supportAsZeroOneVector());
+	}
+	return ret;
+}
+
+
+Polynomial Polynomial::embeddedInto(PolynomialRing const &r2, list<int> const *chosenVariables)const
 {
   Polynomial q(r2);
 
+  if(chosenVariables)
+    {
+      assert(chosenVariables->size()==r2.getNumberOfVariables());
+    }
+
   for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
     {
       IntegerVector v=i->first.exponent;
-      v.resize(r2.getNumberOfVariables());
+      if(chosenVariables)
+	v=v.subvector(*chosenVariables);
+      else
+	v.resize(r2.getNumberOfVariables());
       FieldElement e=i->second;
       q+=Term(e,Monomial(r2,v));
     }
   if(isMarked())
     {
-      IntegerVector m=marked.m.exponent;;
-      m.resize(r2.getNumberOfVariables());
+      IntegerVector m=marked.m.exponent;
+      //AsciiPrinter(Stderr)<<*this<<"\n";
+      if(chosenVariables)
+	m=m.subvector(*chosenVariables);
+      else
+	m.resize(r2.getNumberOfVariables());
       q.mark(Monomial(r2,m));
     }
   //  q.marked.m.exponent.resize(r2.getNumberOfVariables());
@@ -617,6 +802,107 @@ int Polynomial::numberOfVariablesInUseConsecutive()const
   return ret;
 }
 
+
+string Polynomial::toString(bool latex/*, bool mathMode*/)const
+{
+  stringstream s;
+  /*
+  if(latex && !mathMode)
+    s << "$";
+  */
+  bool first=true;
+
+  if(terms.empty())
+    {
+      s << "0";
+      return s.str();
+    }
+  // If the polynomial has a marked term it is written first
+  //   printString("_");
+  IntegerVector e=getMarked().m.exponent;
+  for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+    if(e==i->first.exponent)
+      {
+	s << i->second.toString(i->first.exponent.isZero(),!first,latex);
+	if((!i->first.exponent.isZero())&&(!i->second.isOne())&&(!(-(i->second)).isOne()))s<<"*";
+	s << i->first.toString(false,false,latex);
+	first=false;
+      }
+  //    printString("_");
+  for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+    if(e!=i->first.exponent)
+      {
+	s << i->second.toString(i->first.exponent.isZero(),!first,latex);
+	if((!i->first.exponent.isZero())&&(!i->second.isOne())&&(!(-(i->second)).isOne()))s<<"*";
+	s << i->first.toString(false,false,latex);
+	/*	printFieldElement(i->second,i->first.exponent.isZero(),!first);
+	if((!i->first.exponent.isZero())&&(!i->second.isOne())&&(!(-(i->second)).isOne()))printString("*");
+	printMonomial(i->first,false,false);*/
+	first=false;
+      }
+  /*  if(latex && !mathMode)
+    s << "$";
+  */
+   return s.str();
+}
+
+
+bool Polynomial::checkExponentVectors()const
+{
+	int n=getRing().getNumberOfVariables();
+	for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+		if(i->first.exponent.v.size()!=n)
+		{
+		//	log1
+			{
+			//	AsciiPrinter(Stderr)<<"Exponent vector length does not match ring\n"<<*this;
+				assert(0);
+			}
+			return false;
+		}
+	return true;
+}
+
+
+double Polynomial::evaluateFloat(FloatVector const &x)const
+{
+	int n=getRing().getNumberOfVariables();
+	assert(x.size()==n);
+	double ret=0;
+
+	for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+	{
+		IntegerVector const v=i->first.exponent;
+		double m=i->second.floatingPointApproximation();
+		for(int i=0;i<n;i++)
+		{
+			for(int e=0;e<v[i];e++)m*=x[i];
+		}
+		ret+=m;
+	}
+	return ret;
+}
+
+
+ComplexNumber Polynomial::evaluateComplex(ComplexVector const &x)const
+{
+	int n=getRing().getNumberOfVariables();
+	assert(x.size()==n);
+	ComplexNumber ret(0,0);
+
+	for(TermMap::const_iterator i=terms.begin();i!=terms.end();i++)
+	{
+		IntegerVector const v=i->first.exponent;
+		ComplexNumber m=i->second.floatingPointApproximation();
+		for(int i=0;i<n;i++)
+		{
+			for(int e=0;e<v[i];e++)m*=x[i];
+		}
+		ret+=m;
+	}
+	return ret;
+}
+
 //-----------------------------------------
 // PolynomialSet
 //-----------------------------------------
@@ -627,18 +913,31 @@ void PolynomialSet::saturate(int variableNum)
     i->saturate(variableNum);
 }
 
-PolynomialSet PolynomialSet::polynomialRingIntersection(int n2)const
+PolynomialSet PolynomialSet::polynomialRingIntersection(PolynomialRing const &newRing, list<int> const *chosenVariables)const
 {
-  PolynomialSet ret(theRing);
+  PolynomialSet ret(newRing);
 
-  for(PolynomialSet::const_iterator i=begin();i!=end();i++)
-    if(i->maximalIndexOfVariableInSupport()<n2)
-      {
-	Polynomial p=*i;
-	p.changeNumberOfVariables(n2);
-	ret.push_back(p);
-      }
-  
+  if(chosenVariables==0)
+  {
+	  for(PolynomialSet::const_iterator i=begin();i!=end();i++)
+		  if(i->maximalIndexOfVariableInSupport()<newRing.getNumberOfVariables())
+		  {
+			  ret.push_back(i->embeddedInto(newRing));
+		  }
+  }
+  else
+  {
+	  int n=theRing.getNumberOfVariables();
+	  IntegerVector allowedVariables(n);
+	  for(list<int>::const_iterator i=chosenVariables->begin();i!=chosenVariables->end();i++)allowedVariables[*i]=1;
+	  for(PolynomialSet::const_iterator i=begin();i!=end();i++)
+	  {
+		  if(i->usedVariables().divides(allowedVariables))
+		  {
+			  ret.push_back(i->embeddedInto(newRing,chosenVariables));
+		  }
+	  }
+  }
   return ret;
 }
 
@@ -666,14 +965,27 @@ void PolynomialSet::changeNumberOfVariables(int n)
 }
 
 
-void PolynomialSet::markAndScale(TermOrder const &termOrder)
+void PolynomialSet::mark(class TermOrder const &termOrder)
 {
   for(iterator i=begin();i!=end();i++)
     i->mark(termOrder);
+}
+
+void PolynomialSet::markAndScale(TermOrder const &termOrder)
+{
+  mark(termOrder);
   scaleMarkedCoefficientsToOne();
 }
 
 
+void PolynomialSet::copyMarkings(PolynomialSet const &g)
+{
+  PolynomialSet::const_iterator j=g.begin();
+  for(PolynomialSet::iterator i=begin();i!=end();i++,j++)
+    i->copyMarking(*j);
+}
+
+
 void PolynomialSet::scaleMarkedCoefficientsToOne()
 {
   for(iterator i=begin();i!=end();i++)
@@ -689,6 +1001,34 @@ bool PolynomialSet::checkMarkings(TermOrder const &termOrder)const
   return true;
 }
 
+
+bool PolynomialSet::containsInClosedGroebnerCone(IntegerVector const &v)const
+{
+  for(const_iterator i=begin();i!=end();i++)
+    {
+      if(i->degree(v)>dotLong(v,i->getMarked().m.exponent))return false;
+    }
+  return true;
+}
+
+
+bool PolynomialSet::isHomogeneous(IntegerVector const &v)const
+{
+  for(const_iterator i=begin();i!=end();i++)
+    {
+      if(!i->isHomogeneous(v))return false;
+    }
+  return true;
+}
+
+
+bool PolynomialSet::isHomogeneous()const
+{
+	IntegerVector v=IntegerVector::allOnes(theRing.getNumberOfVariables());
+	return isHomogeneous(v);
+}
+
+
 void PolynomialSet::unionPolynomial(const Polynomial &p)
 {
   const_iterator j;
@@ -712,20 +1052,27 @@ bool PolynomialCompare::operator()(const Polynomial &a, const Polynomial &b)cons
   if(b.terms.size()<a.terms.size())return false;
 
   TermMap::const_iterator i=a.terms.begin();
-  for(TermMap::const_iterator j=b.terms.begin();j!=b.terms.end();j++)
+  for(TermMap::const_iterator j=b.terms.begin();j!=b.terms.end();j++,i++)
     {
       if(LexicographicTermOrder()(i->first.exponent,j->first.exponent))return true;
       if(LexicographicTermOrder()(j->first.exponent,i->first.exponent))return false;
-      i++;
     }
   if((a-b).isZero())return false;
   if((a+b).isZero())return false; // we need some way of comparing field elements
 
-  AsciiPrinter(Stderr).printPolynomial(a);
+/*  AsciiPrinter(Stderr).printPolynomial(a);
   fprintf(Stderr,"\n");
   AsciiPrinter(Stderr).printPolynomial(b);
+*/
 
-  assert("Polynomial compare must be improved to handle this case"==0);
+  i=a.terms.begin();
+  for(TermMap::const_iterator j=b.terms.begin();j!=b.terms.end();j++,i++)
+    {
+      int s=(i->second - j->second).sign();//TODO: Make this work for fields which are not ordered
+      if(s==1)return false;
+      if(s==-1)return true;
+    }
+//  assert("Polynomial compare must be improved to handle this case"==0);
   return false;
 }
 
@@ -753,6 +1100,12 @@ bool PolynomialCompareMarkedTermsReverse::operator()(const Polynomial &a, const
   return termOrder(b.getMarked().m.exponent,a.getMarked().m.exponent);
 }
 
+bool PolynomialCompareNumberOfTermsStable::operator()(const Polynomial &a, const Polynomial &b)const
+{
+  return a.numberOfTerms()<b.numberOfTerms();
+}
+
+
 
 int PolynomialSet::totalDegree()const
 {
@@ -769,6 +1122,11 @@ void PolynomialSet::sort_()
   sort(PolynomialCompare());
 }
 
+void PolynomialSet::simplestPolynomialsFirst()
+{
+  sort(PolynomialCompareNumberOfTermsStable());
+}
+
 int PolynomialSet::numberOfVariablesInRing()const
 {
   return theRing.getNumberOfVariables();
@@ -788,7 +1146,7 @@ bool PolynomialSet::isEqualTo(PolynomialSet const &a)const
   if(a.size()!=size())return false;
 
   PolynomialSet::const_iterator j=begin();
-  
+
   for(PolynomialSet::const_iterator i=a.begin();i!=a.end();i++)
     {
       if(!(*i-*j).isZero())return false;
@@ -798,6 +1156,12 @@ bool PolynomialSet::isEqualTo(PolynomialSet const &a)const
 }
 
 
+bool PolynomialSet::isUnitIdeal()const
+{
+  return division(theRing.one(),*this,StandardGradedLexicographicTermOrder()).isZero();
+}
+
+
 IntegerVector PolynomialSet::exponentsSum()const
 {
   IntegerVector sum(numberOfVariablesInRing());
@@ -838,6 +1202,16 @@ bool PolynomialSet::isMarked()const
 }
 
 
+PolynomialSet PolynomialSet::torusAct(FieldVector const &w)const
+{
+  PolynomialSet ret(theRing);
+  for(const_iterator i=begin();i!=end();i++)
+    ret.push_back(i->torusAct(w));
+
+  return ret;
+}
+
+
 PolynomialSet PolynomialSet::homogenization(PolynomialRing const &newRing, IntegerVector const *w)const
 {
   PolynomialSet ret(newRing);
@@ -848,6 +1222,17 @@ PolynomialSet PolynomialSet::homogenization(PolynomialRing const &newRing, Integ
 }
 
 
+PolynomialSet PolynomialSet::multiDeHomogenization()const
+{
+  int n=getRing().getNumberOfVariables();
+  FieldMatrix m=integerMatrixToFieldMatrix(rowsToIntegerMatrix(wallInequalities(*this),n),Q);
+  m.reduce();
+  list<int> toKeep=m.pivotColumns();
+  PolynomialRing R2=PolynomialRing(getRing().getField(),toKeep.size());
+  return embeddedInto(R2,&toKeep);
+}
+
+
 PolynomialSet PolynomialSet::deHomogenization()const
 {
   PolynomialSet ret(theRing);
@@ -859,6 +1244,17 @@ PolynomialSet PolynomialSet::deHomogenization()const
 }
 
 
+PolynomialSet PolynomialSet::deHomogenizationInSameRing()const
+{
+  PolynomialSet ret(theRing);
+
+  for(const_iterator i=begin();i!=end();i++)
+    ret.push_back(i->deHomogenizationInSameRing());
+
+  return ret;
+}
+
+
 bool PolynomialSet::isValid()const
 {
   if(size()!=0)
@@ -872,11 +1268,11 @@ bool PolynomialSet::isValid()const
 }
 
 
-PolynomialSet PolynomialSet::embeddedInto(PolynomialRing const &r2)const
+PolynomialSet PolynomialSet::embeddedInto(PolynomialRing const &r2, list<int> const *chosenVariables)const
 {
   PolynomialSet ret(r2);
   for(const_iterator i=begin();i!=end();i++)
-    ret.push_back(i->embeddedInto(r2));
+    ret.push_back(i->embeddedInto(r2,chosenVariables));
 
   return ret;
 }
@@ -894,3 +1290,56 @@ int PolynomialSet::numberOfVariablesInUseConsecutive()const
 
   return ret;
 }
+
+
+int PolynomialSet::totalNumberOfTerms()const
+{
+  int ret=0;
+  for(const_iterator i=begin();i!=end();i++)
+    ret+=i->numberOfTerms();
+  return ret;
+}
+
+
+FloatVector PolynomialSet::evaluateFloat(FloatVector const &x)const
+{
+	assert(x.size()==numberOfVariablesInRing());
+	FloatVector ret(size());
+	int I=0;
+	for(const_iterator i=begin();i!=end();i++,I++)
+		ret[I]=i->evaluateFloat(x);
+	return ret;
+}
+
+
+ComplexVector PolynomialSet::evaluateComplex(ComplexVector const &x)const
+{
+	assert(x.size()==numberOfVariablesInRing());
+	ComplexVector ret(size());
+	int I=0;
+	for(const_iterator i=begin();i!=end();i++,I++)
+		ret[I]=i->evaluateComplex(x);
+	return ret;
+}
+
+
+void PolynomialSet::removeZeros()
+{
+  for(PolynomialSet::iterator i=begin();i!=end();i++)
+    {
+      if(i->isZero())
+        {
+          PolynomialSet::iterator j=i;
+          j++;
+          erase(i);
+          j--;
+          i=j;
+        }
+    }
+}
+
+
+void PolynomialSet::removeDuplicates()
+{
+  sort_();
+}
