@@ -15,3 +15,13 @@ void Term::operator*=(const Term &t)
   c*=t.c;
 }
 
+Term Term::operator/(const Term &b)const
+{
+  return Term(c*b.c.inverse(),Monomial(theRing,m.exponent-b.m.exponent));
+}
+
+Term Term::operator/(const Monomial &b)const
+{
+  return Term(c,Monomial(theRing,m.exponent-b.exponent));
+}
+
